//	Array2D.h - A simple 2d version of std::array.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ARRAY2D_H_
#define ARRAY2D_H_

#include "c0xHeaders.h"

///	A simple 2d version of std::array.
template<typename T, int YSize, int XSize>
class Array2D
{
  public:
	///	Default constructor.
	Array2D() {};
	///	Constructor which populates the array with val.
	Array2D(T val)
	{
		int i;

		for(i=0;i<(YSize*XSize);++i)
			data[i] = val;
	};
	///	Copy constructor.
	Array2D(const Array2D& other)
	{
		*this = other;
	};
	///	Destructor.
	~Array2D() {};

	///	Returns the indexed element of the array.
	T *operator[](int index)
	{
		return &(data[index * XSize]);
	};
	///	Returns the indexed element of the array.
	T const * const operator[](int index) const
	{
		return &(data[index * XSize]);
	};

	///	Performs a deep copy of the passed-in Array2D.
	Array2D& operator=(const Array2D& other)
	{
		int x, y;

		for(y=0;y<YSize;++y)
		{
			for(x=0;x<XSize;++x)
				(*this)[y][x] = other[y][x];
		}

		return *this;
	};

	///	Returns the number of elements on the x-axis of the array.
	int getSizeX() const {return XSize;};
	///	Returns the number of elements on the y-axis of the array.
	int getSizeY() const {return YSize;};
  private:
	///	The array data.
	array<T, (YSize*XSize)> data;
};

#endif
