//	Array2DDrawer.h - Simple class used to draw Array2Ds.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ARRAY2DDRAWER_H_
#define ARRAY2DDRAWER_H_

#include "drawing/Drawer.h"
#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Array2D.h"


///	Simple class used to draw Array2Ds.
template<int YSize, int XSize>
class Array2DDrawer
{
  public:
	///	Constructor.
	Array2DDrawer() {};
	///	Destructor.
	~Array2DDrawer() {};

	///	Draws the array, centred on pos.
	void draw(shared_ptr<Drawer>& d,
			  const Array2D<bool, YSize, XSize>& arr,
			  float blobSize,
			  const TwoFloats& pos,
			  const ThreeFloats& col,
			  float alpha = 1.0f)
	{
		int x, y;
		TwoFloats tempPos;

		tempPos.y = pos.y - (blobSize * 32.0f * 0.5f) - (YSize * blobSize * 0.5f * 32.0f);

		for(y=0;y<YSize;++y)
		{
			tempPos.x = pos.x - (blobSize * 32.0f * 0.5f) - (XSize * blobSize * 0.5f * 32.0f);

			for(x=0;x<XSize;++x)
			{
				if(arr[y][x])
				{
					d->drawImage(L"Blob",
								 tempPos,
								 TwoFloats(blobSize),
								 col,
								 alpha);
				}
				tempPos.x += (blobSize * 0.5f * 32.0f);
			}
			tempPos.y += (blobSize * 0.5f * 32.0f);
		}
	};
};

#endif
