//	BeatingSine.cpp - A beating sine tone.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BeatingSine.h"
#include "MiscHelper.h"
#include "Random.h"

//------------------------------------------------------------------------------
BeatingSine::BeatingSine():
amplitude(0.0f),
ampTweener(amplitude, amplitude, 0.001f)
{
	int i;
	float offset;

	for(i=0;i<NumTones;++i)
	{
		tones[i].x0 = 1.0f;
		tones[i].x1 = 0.0f;
		offset = (Random::getGlobal().getRandomFloat() * 8.0f) - 4.0f;
		tones[i].coeff = 2.0f*sinf(NIALL_PI*(440.0f+offset)/44100.0f);
	}
}

//------------------------------------------------------------------------------
BeatingSine::~BeatingSine()
{

}

//------------------------------------------------------------------------------
void BeatingSine::getAudio(float *buffer, int numSamples)
{
	int i, j;

	for(j=0;j<numSamples;++j)
	{
		if(!ampTweener.getFinished())
			ampTweener.tick();

		for(i=0;i<NumTones;++i)
		{
			tones[i].x0 -= (tones[i].coeff*tones[i].x1);
			tones[i].x1 += (tones[i].coeff*tones[i].x0);

			buffer[(j*2)] += tones[i].x1 * amplitude;
			buffer[(j*2)+1] += tones[i].x1 * amplitude;
		}
	}
}

//------------------------------------------------------------------------------
void BeatingSine::setPitch(float freq)
{
	int i;
	float offset;

	for(i=0;i<NumTones;++i)
	{
		tones[i].x0 = 1.0f;
		tones[i].x1 = 0.0f;
		offset = (Random::getGlobal().getRandomFloat() * 8.0f) - 4.0f;
		tones[i].coeff = 2.0f*sinf(NIALL_PI*(freq+offset)/44100.0f);
	}
}

//------------------------------------------------------------------------------
void BeatingSine::setAmplitude(float amp)
{
	if(amplitude == 0.0f)
		amplitude += 0.001f;
	ampTweener.reset(amp);
}
