//	BeatingSine.h - A beating sine tone.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BEATINGSINE_H_
#define BEATINGSINE_H_

#include "Tweener.h"

///	A beating sine tone.
class BeatingSine
{
  public:
	///	Constructor.
	BeatingSine();
	///	Destructor.
	~BeatingSine();

	///	Returns the audio for this sine tone.
	void getAudio(float *buffer, int numSamples);

	///	Sets the tone's base pitch.
	void setPitch(float freq);
	///	Sets the tone's amplitude.
	void setAmplitude(float amp);

	///	Returns true if the tone's amplitude is greater than 0.
	bool getActive() const {return (amplitude > 0.0f);};
  private:
	///	Simple struct representing a single sine tone.
	struct Tone
	{
		///	For the sine generation.
		float x0;
		///	For the sine generation.
		float x1;
		///	For the sine generation.
		float coeff;
	};
	///	How many tones we use.
	enum
	{
		NumTones = 2
	};

	///	The two tones we use.
	Tone tones[NumTones];

	///	The amplitude of the tones.
	float amplitude;
	///	Used to ramp the tone's amplitude.
	Tweener<float> ampTweener;
};

#endif
