//	BlobGlobalData.cpp - BlobVaders global data.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BlobGlobalData.h"
#include "Vader.h"

#include "projectiles/CollidingBullet.h"
#include "projectiles/GrenadeBullet.h"
#include "projectiles/FriendlyFire.h"
#include "projectiles/HomingBullet.h"
#include "projectiles/BasicBullet.h"
#include "projectiles/Mine.h"

#include "damageActions/ExtraHealth.h"
#include "damageActions/Retaliate.h"

#include "killActions/Explode4.h"
#include "killActions/Explode3.h"
#include "killActions/Explode2.h"
#include "killActions/Explode.h"
#include "killActions/Split.h"

//------------------------------------------------------------------------------
BlobGlobalData::BlobGlobalData():
flash(true),
shake(true),
numFleetsDefeated(0),
currentFleetIndex(0),
difficulty(1)
{
	//Register all VaderProjectiles and VaderDamageActions with VaderFactory.
	vaderFactory.registerProjectile<BasicBullet>(1,
												 L"Bullet",
												 L"Kills player on touch.");
	vaderFactory.registerProjectile<CollidingBullet>(1,
													 L"Fragile Bullet",
													 L"Kills player; can be shot out.");
	vaderFactory.registerProjectile<FriendlyFire>(1,
												  L"Friendly Fire",
												  L"Kills player and enemies; invincible.");
	vaderFactory.registerProjectile<HomingBullet>(2,
												  L"Homing Bullet",
												  L"Homes in on player.");
	vaderFactory.registerProjectile<GrenadeBullet>(2,
												   L"Grenade",
												   L"Explodes into Fragile Bullets on death.");
	vaderFactory.registerProjectile<Mine>(2,
										  L"Mine",
										  L"Holds position, explodes on contact.");

	vaderFactory.registerDamageAction<ExtraHealth>(2);
	vaderFactory.registerDamageAction<Retaliate>(2);

	vaderFactory.registerKillAction<Explode>(1);
	vaderFactory.registerKillAction<Explode4>(1);
	vaderFactory.registerKillAction<Explode2>(2);
	vaderFactory.registerKillAction<Split>(2);
	vaderFactory.registerKillAction<Explode3>(3);
}

//------------------------------------------------------------------------------
BlobGlobalData::~BlobGlobalData()
{

}

//------------------------------------------------------------------------------
void BlobGlobalData::setFlashingBackground(bool val)
{
	flash = val;
}

//------------------------------------------------------------------------------
void BlobGlobalData::setScreenShake(bool val)
{
	shake = val;
}

//------------------------------------------------------------------------------
void BlobGlobalData::resetGame()
{
	numFleetsDefeated = 0;
	storedPlanets.clear();
	storedFleets.clear();
	currentFleetIndex = 0;
	vaderEncyclopedia.clear();
	vaderFactory.initialise();
	difficulty = 1;
}

//------------------------------------------------------------------------------
void BlobGlobalData::incNumFleetsDefeated()
{
	++numFleetsDefeated;
	storedFleets.erase(storedFleets.begin()+currentFleetIndex);
}

//------------------------------------------------------------------------------
void BlobGlobalData::storePlanets(const std::vector<shared_ptr<Planet> >& planets)
{
	unsigned int i;

	storedPlanets.clear();
	for(i=0;i<planets.size();++i)
		storedPlanets.push_back(planets[i]);
}

//------------------------------------------------------------------------------
void BlobGlobalData::storeFleets(const std::vector<shared_ptr<Fleet> >& fleets)
{
	unsigned int i;

	storedFleets.clear();
	for(i=0;i<fleets.size();++i)
		storedFleets.push_back(fleets[i]);
}

//------------------------------------------------------------------------------
void BlobGlobalData::setCurrentFleetIndex(int index)
{
	currentFleetIndex = index;
}

//------------------------------------------------------------------------------
void BlobGlobalData::setPlayerPosition(const TwoFloats& pos)
{
	playerPosition = pos;
}

//------------------------------------------------------------------------------
void BlobGlobalData::incrementDifficulty()
{
	++difficulty;
}

//------------------------------------------------------------------------------
void BlobGlobalData::addToEncyclopedia(const shared_ptr<Vader>& vader)
{
	unsigned int i;
	bool found = false;

	for(i=0;i<vaderEncyclopedia.size();++i)
	{
		if(vaderEncyclopedia[i]->getVaderName() == vader->getVaderName())
		{
			found = true;
			break;
		}
	}
	if(!found)
		vaderEncyclopedia.push_back(vader);
}
