//	BlobVader.cpp - The basic BlobVader enemy.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Random.h"
#include "BlobVader.h"
#include "drawing/Drawer.h"

//------------------------------------------------------------------------------
BlobVader::BlobVader(const TwoFloats& startPos):
position(startPos),
scale(0.625f),
scaleTween(scale, scale, 0.075f),
initialTween(position, TwoFloats(startPos.x, startPos.y+448.0f), 0.01f),
stepTween(position, TwoFloats(startPos), 0.05f),
dead(false),
deadCount(1.0f),
deadTween(deadCount, 0.0f, 0.05f)
{
	while((colour == 0.0f) || (colour == 1.0f))
	{
		colour.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		colour.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		colour.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
	}
	if(colour.x > 0.0f)
		coronaColour.x = colour.x - /*Random::getGlobal().getRandomFloat() **/ 0.1f;
	else
		coronaColour.x = colour.x + /*Random::getGlobal().getRandomFloat() **/ 0.1f;
	if(colour.y > 0.0f)
		coronaColour.y = colour.y - /*Random::getGlobal().getRandomFloat() **/ 0.1f;
	else
		coronaColour.y = colour.y + /*Random::getGlobal().getRandomFloat() **/ 0.1f;
	if(colour.z > 0.0f)
		coronaColour.z = colour.z - /*Random::getGlobal().getRandomFloat() **/ 0.1f;
	else
		coronaColour.z = colour.z + /*Random::getGlobal().getRandomFloat() **/ 0.1f;

	wingMotion = Random::getGlobal().getRandomFloat();
	initialTween.setDelay(Random::getGlobal().getRandomFloat());
	stepTween.setIndex(1.0f);
	createBlobArray();
}

//------------------------------------------------------------------------------
BlobVader::~BlobVader()
{

}

//------------------------------------------------------------------------------
void BlobVader::update()
{
	if(!initialTween.getFinished())
		initialTween.tick();
	else if(!stepTween.getFinished())
		stepTween.tick();

	if(!scaleTween.getFinished())
		scaleTween.tick();

	if(dead)
	{
		if(!deadTween.getFinished())
			deadTween.tick();
	}
}

//------------------------------------------------------------------------------
void BlobVader::draw(DrawerPtr d, const ThreeFloats& leftCol, const ThreeFloats& rightCol)
{
	float tempf;
	unsigned int i, j;
	TwoFloats tempPos;
	TwoFloats tempDir;
	ThreeFloats tempCol;

	tempf = (position.x - 128.0f)/(1280.0f-256.0f);
	tempCol = (leftCol * (1.0f-tempf)) + (rightCol * tempf);

	if(!dead)
	{
		tempf = sinf(wingMotion*2.0f*NIALL_PI);
		for(i=0;i<4;++i)
		{
			for(j=0;j<wings[i].size();++j)
			{
				tempPos = position+blobArray[wings[i][j]]-TwoFloats(16.0f*scale);
				tempPos.y += tempf * (((float)(4-i)/4.0f) * 8.0f);

				d->drawImage(L"Blob",
							 tempPos,
							 TwoFloats(scale),
							 tempCol);
			}
		}
	}
	else
	{
		const TwoFloats origin;

		for(i=0;i<blobArray.size();++i)
		{
			tempDir = (blobArray[i]-origin) * blobArray[i].getDistance(origin);
			tempDir.y += 1000.0f;
			tempDir = tempDir.normal();

			blobArray[i] += tempDir * 4.0f * (1.0f-deadCount);

			tempPos = position+blobArray[i]-TwoFloats(16.0f*scale);

			d->drawImage(L"Blob",
						 tempPos,
						 TwoFloats(scale),
						 tempCol,
						 deadCount);
		}
	}

	wingMotion += 0.01f;
	if(wingMotion > 1.0f)
		wingMotion -= 1.0f;
}

//------------------------------------------------------------------------------
void BlobVader::setPosition(const TwoFloats& pos)
{
	TwoFloats desiredPos(pos);

	position = pos;
	desiredPos.y += 720.0f;
	initialTween.reset(desiredPos);
}

//------------------------------------------------------------------------------
void BlobVader::stepLeft()
{
	TwoFloats tempPos(position);

	tempPos.x -= (1280.0f-256.0f)/8.0f;
	stepTween.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	stepTween.reset(tempPos);
}

//------------------------------------------------------------------------------
void BlobVader::stepRight()
{
	TwoFloats tempPos(position);

	tempPos.x += (1280.0f-256.0f)/8.0f;
	stepTween.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	stepTween.reset(tempPos);
}

//------------------------------------------------------------------------------
void BlobVader::stepDown()
{
	TwoFloats tempPos(position);

	tempPos.y += (640-256.0f)/4.0f;
	stepTween.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	stepTween.reset(tempPos);
}

//------------------------------------------------------------------------------
void BlobVader::scaleUp()
{
	scale = 1.25f;
	scaleTween.reset(0.625f);
}

//------------------------------------------------------------------------------
void BlobVader::setDead()
{
	dead = true;
	//scaleTween.reset(0.33f);
}

//------------------------------------------------------------------------------
void BlobVader::createBlobArray()
{
	int x, y;
	TwoFloats tempPos;
	int blobCount = 0;
	bool usedPos[7][7];
	float chance = 0.75f;

	for(y=0;y<7;++y)
	{
		for(x=0;x<7;++x)
			usedPos[y][x] = false;
	}

	usedPos[3][3] = true;
	++blobCount;
	blobArray.push_back(tempPos);
	wings[3].push_back(0);

	x = 3;
	for(y=2;y>=0;--y)
	{
		if(Random::getGlobal().getRandomFloat() > chance)
			break;
		chance -= 0.25f;

		usedPos[y][x] = true;
		++blobCount;
		tempPos.x = ((float)x-3.0f) * 10.0f;
		tempPos.y = ((float)y-3.0f) * 10.0f;
		blobArray.push_back(tempPos);
		wings[3].push_back(blobArray.size()-1);
	}
	for(y=4;y<7;++y)
	{
		if(Random::getGlobal().getRandomFloat() > chance)
			break;
		chance -= 0.25f;

		usedPos[y][x] = true;
		++blobCount;
		tempPos.x = ((float)x-3.0f) * 10.0f;
		tempPos.y = ((float)y-3.0f) * 10.0f;
		blobArray.push_back(tempPos);
		wings[3].push_back(blobArray.size()-1);
	}

	while(blobCount < 16)
	{
		for(x=2;x>=0;--x)
		{
			for(y=2;y>=0;--y)
			{
				if(usedPos[y][x])
					continue;

				if((usedPos[y][x+1] && (Random::getGlobal().getRandomFloat() > 0.5f)) ||
				   (Random::getGlobal().getRandomFloat() > 0.75f))
				{
					if((x < 2) && !usedPos[y][x+1] /*&& !usedPos[y][x+2]*/ && !usedPos[y+1][x])
						continue;

					usedPos[y][x] = true;
					++blobCount;

					tempPos.x = ((float)x-3.0f) * 10.0f;
					tempPos.y = ((float)y-3.0f) * 10.0f;
					blobArray.push_back(tempPos);
					wings[x].push_back(blobArray.size()-1);

					usedPos[y][6-x] = true;
					++blobCount;

					tempPos.x = ((float)(6-x)-3.0f) * 10.0f;
					blobArray.push_back(tempPos);
					wings[x].push_back(blobArray.size()-1);
				}
			}

			for(y=4;y<7;++y)
			{
				if(usedPos[y][x])
					continue;

				if((usedPos[y][x+1] && (Random::getGlobal().getRandomFloat() > 0.5f)) ||
				   (Random::getGlobal().getRandomFloat() > 0.75f))
				{
					if((x < 2) && !usedPos[y][x+1] /*&& !usedPos[y][x+2]*/ && !usedPos[y-1][x])
						continue;

					usedPos[y][x] = true;
					++blobCount;

					tempPos.x = ((float)x-3.0f) * 10.0f;
					tempPos.y = ((float)y-3.0f) * 10.0f;
					blobArray.push_back(tempPos);
					wings[x].push_back(blobArray.size()-1);

					usedPos[y][6-x] = true;
					++blobCount;

					tempPos.x = ((float)(6-x)-3.0f) * 10.0f;
					blobArray.push_back(tempPos);
					wings[x].push_back(blobArray.size()-1);
				}
			}
		}
	}

	/*for(x=0;x<3;++x)
	{
		for(y=0;y<7;++y)
		{
			if(Random::getGlobal().getRandomFloat() > 0.66f)
			{
				tempPos.x = ((float)x-3.0f) * 10.0f;
				tempPos.y = ((float)y-3.0f) * 10.0f;
				blobArray.push_back(tempPos);

				tempPos.x = ((float)(6-x)-3.0f) * 10.0f;
				blobArray.push_back(tempPos);
			}
		}
	}
	x = 3;
	for(y=0;y<7;++y)
	{
		if(Random::getGlobal().getRandomFloat() > 0.66f)
		{
			tempPos.x = ((float)x-3.0f) * 10.0f;
			tempPos.y = ((float)y-3.0f) * 10.0f;
			blobArray.push_back(tempPos);
		}
	}*/
}
