//	BlobVader.h - The basic BlobVader enemy.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BLOBVADER_H_
#define BLOBVADER_H_

#include "Tweener.h"
#include "TwoFloats.h"
#include "c0xHeaders.h"
#include "ThreeFloats.h"

#include <vector>

class Drawer;
typedef shared_ptr<Drawer> DrawerPtr;

///	The basic BlobVader enemy.
class BlobVader
{
  public:
	///	Constructor.
	BlobVader(const TwoFloats& startPos);
	///	Destructor.
	~BlobVader();

	///	Updates the BlobVader's position.
	void update();
	///	Draws the BlobVader.
	void draw(DrawerPtr d, const ThreeFloats& leftCol, const ThreeFloats& rightCol);

	///	Sets the BlobVader's position.
	void setPosition(const TwoFloats& pos);
	///	Returns the BlobVader's position.
	const TwoFloats& getPosition() const {return position;};

	///	Sets the BlobVader to step left.
	void stepLeft();
	///	Sets the BlobVader to step right.
	void stepRight();
	///	Sets the BlobVader to step down.
	void stepDown();

	///	Tells the BlobVader to briefly scale up.
	void scaleUp();

	///	Tells the BlobVader it is dead.
	void setDead();
	///	Returns true if the BlobVader is dead.
	bool getDead() const {return dead;};
	///	Returns true if the BlobVader can be deleted.
	bool getFinished() const {return deadCount <= 0.0f;};
	///	Returns true if the BlobVader is still falling from the sky.
	bool getInvulnerable() const {return !initialTween.getFinished();};
  private:
	///	Helper method. Creates the BlobVader's blobArray.
	void createBlobArray();

	///	The BlobVader's position.
	TwoFloats position;
	///	The BlobVader's colour.
	ThreeFloats colour;
	///	The BlobVader's colour.
	ThreeFloats coronaColour;

	///	The arrangement of blobs which make up the BlobVader.
	std::vector<TwoFloats> blobArray;
	///	Used to make the BlobVader's 'wings' move.
	float wingMotion;
	///	Used to make the BlobVader's 'wings' move.
	std::vector<int> wings[4];

	///	The scale of the BlobVader's blobs.
	float scale;
	///	Used to tween the scale of the BlobVader's blobs.
	BounceTweener<float> scaleTween;

	///	Used to tween the BlobVader down from the top of the screen.
	BounceTweener<TwoFloats> initialTween;
	///	Used to tween the BlobVader to it's next position (via stepLeft() etc.).
	STweener<TwoFloats> stepTween;

	///	True if the BlobVader is dead.
	bool dead;
	///	Used to split the BlobVader apart when it's killed.
	float deadCount;
	///	Tweener for deadCount.
	LogTweener<float> deadTween;
};

#endif
