//	BlobVaderSounder.cpp - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BlobVaderSounder.h"
#include "UTF8File.h"
#include "Random.h"

#include "synth/KickSynth.h"
#include "synth/ClapSynth.h"
#include "synth/HatSynth.h"
#include "synth/CrashSynth.h"
#include "FleetDrone.inl"
#include "SunDrone.inl"
#include "BlobBass.inl"
#include "BlobPeow.inl"
#include "BlobPad.inl"
#include "BlobPew.inl"
#include "BlobBop.inl"
#include "BlobHi.inl"

#include <cassert>
#include <fstream>
#include <sstream>
#include <string>

using namespace std;

//------------------------------------------------------------------------------
/*float pitches[8] = {(1975.5332050245f*0.25f),
					(1567.981743927f*0.25f),
					(1318.5102276515f*0.25f),
					(1174.65907166965f*0.25f),
					(987.76660251225f*0.25f),
					(783.9908719635f*0.25f),
					(659.25511382575f*0.25f),
					(523.2511306012f*0.25f)};*/

//------------------------------------------------------------------------------
BlobVaderSounder::BlobVaderSounder():
Sounder(),
currentPadStep(0),
playing(false),
action(0.0f),
callback(0)
{
	int i, j;

	for(i=0;i<2;++i)
	{
		for(j=0;j<NumSteps;++j)
		{
			steps[i][j] = 0.0f;
			stepDecays[i][j] = 0.0f;
		}
	}

	for(i=0;i<4;++i)
		padSteps[i] = 0.0f;

	/*steps[0][0] = 523.2511306012f*0.5f;
	steps[0][4] = 523.2511306012f*0.5f;
	steps[0][8] = 783.9908719635f*0.5f;
	steps[0][12] = 659.25511382575f*0.5f;

	steps[1][0] = 523.2511306012f*0.5f;
	steps[1][1] = 523.2511306012f;
	steps[1][2] = 523.2511306012f*0.5f;
	steps[1][3] = 523.2511306012f;
	steps[1][4] = 523.2511306012f*0.5f;
	steps[1][5] = 523.2511306012f;
	steps[1][6] = 523.2511306012f*0.5f;
	steps[1][7] = 523.2511306012f;
	steps[1][8] = 523.2511306012f*0.5f;
	steps[1][9] = 523.2511306012f;
	steps[1][10] = 523.2511306012f*0.5f;
	steps[1][11] = 523.2511306012f;
	steps[1][12] = 523.2511306012f*0.5f;
	steps[1][13] = 523.2511306012f;
	steps[1][14] = 523.2511306012f*0.5f;
	steps[1][15] = 523.2511306012f;*/

	synths[0].loadParameters(BlobBass::numParameters, BlobBass::parameters);
	synths[1].loadParameters(BlobHi::numParameters, BlobHi::parameters);
	synths[2].loadParameters(BlobPad::numParameters, BlobPad::parameters);

	drums[0] = shared_ptr<Synthesizer>(new KickSynth());
	drums[1] = shared_ptr<Synthesizer>(new ClapSynth());
	drums[2] = shared_ptr<Synthesizer>(new HatSynth());
	drums[3] = shared_ptr<Synthesizer>(new CrashSynth());

	sfxSynths[0].loadParameters(BlobPeow::numParameters, BlobPeow::parameters);
	sfxSynths[0].setParameter("level", 0.5f);
	sfxSynths[1].loadParameters(BlobPew::numParameters, BlobPew::parameters);
	sfxSynths[1].setParameter("level", 0.5f);
	sfxSynths[2].loadParameters(BlobBop::numParameters, BlobBop::parameters);
	sfxSynths[2].setParameter("level", 2.0f);

	cursor.registerCallback(this);
	cursor.setTempo(66.0f);

	hiDelay.setDelay((66.0f/60.0f)/8.0f);
	hiDelay.setFeedback(0.66f);

	loadPlanetPitches();

	brushStrokes.setFrequency(planetPitches[6]);
	brushDelay.setDelay(1.0f);
	brushDelay.setFeedback(0.66f);

	for(i=0;i<SunDrone::numParameters;++i)
		sunSynth.setParameter(sunSynth.getParameterName(i), SunDrone::parameters[i], true);

	for(i=0;i<NumSines*2;++i)
	{
		for(j=0;j<FleetDrone::numParameters;++j)
			fleetSynths[i].setParameter(fleetSynths[i].getParameterName(i), FleetDrone::parameters[i], true);
		fleetSynths[i].setParameter("level", 0.5f);
	}
}

//------------------------------------------------------------------------------
BlobVaderSounder::~BlobVaderSounder()
{

}

//------------------------------------------------------------------------------
void BlobVaderSounder::getAudio(float *input, float *output, int numSamples)
{
	int i, j;

	assert((numSamples*2) < 88200);

	for(i=0;i<(numSamples*2);++i)
	{
		delayBuffer[i] = 0.0f;
		brushBuffer[i] = 0.0f;
	}

	if(playing)
		cursor.audioBlock(numSamples);

	for(i=0;i<4;++i)
	{
		if(drums[i]->getActive())
			drums[i]->getAudio(output, numSamples);
	}
	for(i=0;i<numSamples;++i)
	{
		output[(i*2)] = tanhf(output[(i*2)] * 2.0f);
		output[(i*2)+1] = tanhf(output[(i*2)+1] * 2.0f);
	}

	synths[0].getAudio(output, numSamples);
	synths[1].getAudio(delayBuffer, numSamples);
	synths[2].getAudio(delayBuffer, numSamples);
	hiDelay.getAudio(delayBuffer, numSamples);
	for(i=0;i<(numSamples*2);++i)
		output[i] += delayBuffer[i];

	sfxSynths[0].getAudio(output, numSamples);
	sfxSynths[1].getAudio(output, numSamples);
	sfxSynths[2].getAudio(output, numSamples);

	//Solar system state sounds.
	for(i=0;i<NumSines;++i)
	{
		if(planetSines[i].getActive())
			planetSines[i].getAudio(output, numSamples);
	}

	if(brushStrokes.getActive())
		brushStrokes.getAudio(brushBuffer, numSamples);
	if(sunSynth.getActive())
		sunSynth.getAudio(brushBuffer, numSamples);
	for(i=0;i<NumSines;++i)
	{
		if(fleetSynths[i].getActive())
			fleetSynths[i].getAudio(brushBuffer, numSamples);
	}

	brushDelay.getAudio(brushBuffer, numSamples);
	for(i=0;i<(numSamples*2);++i)
		output[i] += brushBuffer[i];

	for(i=0;i<2;++i)
	{
		for(j=0;j<NumSteps;++j)
		{
			if(stepDecays[i][j] > 0.0f)
			{
				stepDecays[i][j] -= 0.01f;
				if(stepDecays[i][j] <= 0.0f)
				{
					stepDecays[i][j] = 0.0f;
					steps[i][j] = 0.0f;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::startSequence()
{
	int i, j;

	for(i=0;i<2;++i)
	{
		for(j=0;j<NumSteps;++j)
		{
			steps[i][j] = 0.0f;
			stepDecays[i][j] = 0.0f;
		}
	}
	for(i=0;i<4;++i)
		padSteps[i] = 0.0f;

	cursor.resetPosition();
	playing = true;
}

//------------------------------------------------------------------------------
void BlobVaderSounder::stopSequence()
{
	playing = false;
}

//------------------------------------------------------------------------------
void BlobVaderSounder::setAmountAction(float val)
{
	action = val;
}

//------------------------------------------------------------------------------
void BlobVaderSounder::addNote(int synth, int cell, int freq)
{
	int prevCell;
	int newFreq;
	float actualFreq;

	prevCell = cell-1;
	if(prevCell < 0)
		prevCell += NumSteps;
	if(synth)
		actualFreq = pitches[freq] * 2.0f;
	else
		actualFreq = pitches[freq];
	if(actualFreq == steps[synth][prevCell])
	{
		/*if(freq > 0)
			newFreq = (freq + ((Random::getGlobal().getRandomFloat() > 0.5f) ? -1 : 1)) % 8;
		else
			newFreq = 1;*/
		newFreq = Random::getGlobal().getRandomInt(7);
	}
	else
		newFreq = freq;

	if(stepDecays[synth][cell] <= 0.0f)
	{
		if(synth)
		{
			stepDecays[synth][cell] = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
			steps[synth][cell] = pitches[newFreq] * 2.0f;
		}
		else
		{
			stepDecays[synth][cell] = 1.0f;
			steps[synth][cell] = pitches[newFreq];
		}
	}
	else if(synth)
	{
		++cell;
		cell %= 16;

		if(stepDecays[synth][cell] <= 0.0f)
		{
			stepDecays[synth][cell] = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
			steps[synth][cell] = pitches[newFreq] * 2.0f;
		}
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::setPadStep(int cell, int freq)
{
	if(freq > -1)
		padSteps[cell] = pitches[freq];
	else
		padSteps[cell] = 0.0f;
}

//------------------------------------------------------------------------------
void BlobVaderSounder::triggerSFX(int synth)
{
	float offset = (Random::getGlobal().getRandomFloat() * 1.5f)+0.5f;

	sfxSynths[synth].triggerNote(220.0f*offset, 0);
}

//------------------------------------------------------------------------------
void BlobVaderSounder::setSineAmplitude(int index, float val)
{
	assert(index < NumSines);

	planetSines[index].setAmplitude(val);
	brushStrokes.setFrequency(planetPitches[index]);
}

//------------------------------------------------------------------------------
void BlobVaderSounder::activateBrushStrokes(bool val)
{
	if(val)
		brushStrokes.setFrequency(planetPitches[6]);
	brushStrokes.setActive(val);
}

//------------------------------------------------------------------------------
void BlobVaderSounder::setBrushStrokesTrail(TwoFloats *trail)
{
	brushStrokes.setTables(trail);
}

//------------------------------------------------------------------------------
void BlobVaderSounder::activateSun(bool val)
{
	if(val && !sunSynth.getActive())
		sunSynth.noteOn(planetPitches[7], 0);
	else if(!val && sunSynth.getActive())
		sunSynth.noteOff(0);
}

//------------------------------------------------------------------------------
void BlobVaderSounder::activateFleet(int index, bool val)
{
	assert(index < 6);

	if(val && !fleetSynths[index].getActive())
	{
		fleetSynths[index].noteOn(planetPitches[5-index], 0);
		fleetSynths[index*2].noteOn(planetPitches[5-index]*0.5f, 0);
	}
	else if(!val && fleetSynths[index].getActive())
	{
		fleetSynths[index].noteOff(0);
		fleetSynths[index*2].noteOff(0);
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::nextStepCallback(int step, int sample)
{
	int i;

	if(!(step%4))
	{
		drums[Kick]->setActive(true, sample, false);

		if((step == 4) || (step == 12))
			drums[Clap]->setActive(true, sample, false);

		if((step == 0) && (action > 0.5f))
			drums[Crash]->setActive(true, sample, false);

		if(callback)
			callback->beatCallback();
	}

	if((step%2) && (action > 0.66f))
		drums[Hat]->setActive(true, sample, false);
	else if(!((step-2)%4) && (action > 0.25f))
		drums[Hat]->setActive(true, sample, true);

	for(i=0;i<2;++i)
	{
		if(steps[i][step] > 0.0f)
			synths[i].triggerNote(steps[i][step], sample);
	}

	if(step == 0)
	{
		if(padSteps[currentPadStep] > 0.0f)
		{
			synths[2].triggerNote(padSteps[currentPadStep], sample);
			synths[2].triggerNote(padSteps[currentPadStep]*2.0f, sample);
		}
		++currentPadStep;
		currentPadStep %= 4;
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::loadPitches()
{
	int index;
	FilePath pitchFile;
	FilePath pitchesDir = FilePath::getDataFilesDir().getChild(L"pitches");

	index = Random::getGlobal().getRandomInt(pitchesDir.getNumChildren());
	pitchFile = pitchesDir.getChild(index);

	if(pitchFile.exists())
	{
		if(pitchFile.getExtension() == L"txt")
		{
			UTF8File phil(pitchFile);

			if(!phil.getFail())
			{
				int i;
				float tempf;
				wstring tempstr;
				wstringstream tempstr2;

				i = 0;
				while(!phil.readLine(tempstr))
				{
					assert(i < 8);

					tempstr2.clear();
					tempstr2.str(L"");
					tempstr2 << tempstr;

					tempstr2 >> tempf;

					pitches[i] = tempf * 0.25f;

					++i;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::loadPlanetPitches()
{
	int i;

	loadPitches();
	for(i=0;i<8;++i)
	{
		planetPitches[i] = pitches[i];
		planetSines[i].setPitch(planetPitches[i]);
	}
}

//------------------------------------------------------------------------------
void BlobVaderSounder::registerCallback(Callback *c)
{
	callback = c;
}
