//	Fleet.cpp - Class representing a vader fleet on the solar system screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Random.h"
#include "Fleet.h"

#include <sstream>

using std::wstringstream;

//------------------------------------------------------------------------------
Fleet::Fleet(const TwoFloats& dest,
			 const TwoFloats& sunPos,
			 float distance):
positionTweener(position, destination, 0.01f),
destination(dest),
colourTweener(colour, colour, 0.01f),
synth(0),
scale(1.0f),
mouseIsOver(false)
{
	TwoFloats dir;
	ThreeFloats tempCol;

	dir = (destination-sunPos).normal();
	position = destination + (dir * distance);
	position.x += (Random::getGlobal().getRandomFloat() * 384.0f)-192.0f;
	position.y += (Random::getGlobal().getRandomFloat() * 384.0f)-192.0f;
	positionTweener.setDelta((0.001f*560.0f)/distance);
	positionTweener.reset(destination);

	while((tempCol == 0.0f) || (tempCol == 1.0f))
	{
		tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
	}
	colour = tempCol;

	while((tempCol == 0.0f) || (tempCol == 1.0f) || (tempCol == colour))
	{
		tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
	}
	colourTweener.reset(tempCol);
}

//------------------------------------------------------------------------------
Fleet::~Fleet()
{

}

//------------------------------------------------------------------------------
void Fleet::update()
{
	if(!positionTweener.getFinished())
		positionTweener.tick();
}

//------------------------------------------------------------------------------
void Fleet::draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos)
{
	wstringstream tempstr;
	TwoFloats tempPos(position-screenPos);

	if(mouseIsOver && (scale < 0.8f))
		scale += 0.005f;
	else if(!mouseIsOver && (scale > 0.66f))
		scale -= 0.005f;

	if(!colourTweener.getFinished())
		colourTweener.tick();
	else
	{
		ThreeFloats tempCol;

		while((tempCol == 0.0f) || (tempCol == 1.0f) || (tempCol == colour))
		{
			tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
			tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
			tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		}
		colourTweener.reset(tempCol);
	}

	if((tempPos > TwoFloats(-48.0f)) &&
		(tempPos < (d->getDrawingAreaSize()+48.0f)))
	{
		tempPos -= 16.0f * scale;

		tempPos.x -= 20.0f * scale;
		tempPos.y -= 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

		tempPos.y += 10.0f * scale;
		tempPos.x -= 30.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

		tempPos.y += 10.0f * scale;
		tempPos.x -= 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

#ifndef DEBUG
		/*if(position != destination)
		{
			tempstr.precision(2);
			tempstr << std::fixed << ((5.0f/560.0f)*(position.getDistance(destination))) << L" days";
			tempPos = position-screenPos;
			tempPos.x -= 192.0f;
			tempPos.y -= 32.0f;
			d->drawText(tempstr.str(),
						L"Font2",
						tempPos,
						384.0f,
						TextLayout::Centre,
						1.0f,
						colour);
		}*/
#endif
	}
	else
	{
		float pulse;
		//TwoFloats dir((position-destination).normal()*32.0f);
		TwoFloats dir((position-(screenPos + (d->getDrawingAreaSize() * 0.5f))).normal() * 8.0f);

		tempPos = (d->getDrawingAreaSize() * 0.5f);

		while((tempPos > TwoFloats(0.0f)) && (tempPos < d->getDrawingAreaSize()))
			tempPos += dir;

		if(tempPos.x < 0.0f)
			tempPos.x = 36.0f;
		else if(tempPos.x > d->getDrawingAreaSize().x)
			tempPos.x = d->getDrawingAreaSize().x-36.0f;

		if(tempPos.y < 0.0f)
			tempPos.y = 36.0f;
		else if(tempPos.y > d->getDrawingAreaSize().y)
			tempPos.y = d->getDrawingAreaSize().y-36.0f;

		//tempPos.x -= 96.0f;
		tempPos -= 32.0f * 0.8f;

#ifndef DEBUG
		/*tempstr.precision(2);
		tempstr << std::fixed << ((5.0f/560.0f)*(position.getDistance(destination))) << L" days";
		d->drawText(tempstr.str(),
					L"Font2",
					tempPos,
					192.0f,
					TextLayout::Centre,
					1.0f,
					colour);*/
#endif
		pulse = sinf(warningPulse * 2.0f * NIALL_PI);
		pulse *= 0.25f;
		pulse += 0.75f;
		d->drawImage(L"VaderWarning",
					 tempPos,
					 TwoFloats(0.8f),
					 ThreeFloats(1.0f, 0.1f, 0.1f),
					 pulse);

		warningPulse += 0.01f;
		if(warningPulse > 1.0f)
			warningPulse -= 1.0f;
	}
}

//------------------------------------------------------------------------------
void Fleet::setDestination(const TwoFloats& pos)
{
	destination = pos;

	positionTweener.setDelta((0.001f*560.0f)/position.getDistance(destination));
	positionTweener.reset(destination);
}

//------------------------------------------------------------------------------
void Fleet::setSynth(int val)
{
	synth = val;
}

//------------------------------------------------------------------------------
void Fleet::mouseOver(bool val)
{
	mouseIsOver = val;
}

//------------------------------------------------------------------------------
TwoFloats Fleet::getLineIntersection(const TwoFloats& line1a,
									 const TwoFloats& line1b,
									 const TwoFloats& line2a,
									 const TwoFloats& line2b)
{
	float tempf;
	float tempf2;
	TwoFloats retval;

	tempf = ((line2b.x-line2a.x)*(line1a.y-line2a.y))-((line2b.y-line2a.y)*(line1a.x-line2a.x));
	tempf2 = ((line2b.y-line2a.y)*(line1b.x-line1a.x))-((line2b.x-line2a.x)*(line1b.y-line1a.y));
	if(tempf2 == 0.0f)
		return retval;
	tempf /= tempf2;

	retval.x = line1a.x + tempf*(line1b.x-line1a.x);
	retval.y = line1a.y + tempf*(line1b.y-line1a.y);

	return retval;
}
