//	Fleet.h - Class representing a vader fleet on the solar system screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FLEET_H_
#define FLEET_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

class Drawer;

///	Class representing a vader fleet on the solar system screen.
class Fleet
{
  public:
	///	Constructor.
	/*!
		\param dest The fleet's intended destination.
		\param sunPos The position of the sun.
		\param distance The distance from the destination at which the fleet
		should be spawned.
	 */
	Fleet(const TwoFloats& dest,
		  const TwoFloats& sunPos,
		  float distance);
	///	Destructor.
	~Fleet();

	///	Moves the fleet.
	void update();
	///	Draws the fleet.
	void draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos);

	///	Updates the fleet's destination.
	void setDestination(const TwoFloats& pos);

	///	Returns the fleet's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the fleet's destination.
	const TwoFloats& getDestination() const {return destination;};
	///	Returns the fleet's colour.
	const ThreeFloats& getColour() const {return colour;};

	///	Sets the fleet's synth index.
	void setSynth(int val);
	///	Returns the fleet's synth index (-1 if it doesn't currently have one).
	int getSynth() const {return synth;};

	///	So we know whether the mouse is over us or not.
	void mouseOver(bool val);
  private:
	///	Helper method. Finds the intersection of 2 lines.
	TwoFloats getLineIntersection(const TwoFloats& line1a,
								  const TwoFloats& line1b,
								  const TwoFloats& line2a,
								  const TwoFloats& line3b);

	///	The fleet's position.
	TwoFloats position;
	///	Tweener used to move the fleet to its destination.
	STweener<TwoFloats> positionTweener;
	///	The fleet's destination.
	TwoFloats destination;

	///	The fleet's colour.
	ThreeFloats colour;
	///	Tweener used to interpolate the fleet's colour.
	STweener<ThreeFloats> colourTweener;

	///	Used to pulse the warning sign.
	float warningPulse;

	///	The fleet's synth index.
	int synth;

	///	The fleet's current scale.
	float scale;
	///	If the mouse is currently over the fleet.
	bool mouseIsOver;
};

#endif
