//	Player.cpp - The player ship.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Player.h"
#include "drawing/Drawer.h"

//------------------------------------------------------------------------------
Player::Player():
position(640.0f, 660.0f),
lastMouse(640.0f, 660.0f),
fireVal(1.0f),
dead(false)
{

}

//------------------------------------------------------------------------------
Player::~Player()
{

}

//------------------------------------------------------------------------------
void Player::update()
{
	float lastX = position.x;

	if(position != lastMouse)
	{
		if(position.getDistance(lastMouse) < 1.0f)
			position == lastMouse;
		else
			position.x += (lastMouse.x-position.x) * 0.2f;
	}

	speed = (position.x - lastX) * 0.0625f;
}

//------------------------------------------------------------------------------
void Player::draw(DrawerPtr d, const ThreeFloats& col)
{
	float tempf = fabsf(speed);
	TwoFloats tempPos(position);

	position.y = d->getDrawingAreaSize().y-60.0f;
	tempPos = position;

	tempPos.x -= 16.0f * fireVal * (0.625f+tempf);
	tempPos.y -= 16.0f * fireVal*0.625f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);

	tempPos.y -= 10.0f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);

	tempPos.x += 10.0f;
	tempPos.y += 10.0f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);
	tempPos.x -= 20.0f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);

	tempPos.x -= 10.0f;
	tempPos.y += 10.0f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);
	tempPos.x += 40.0f;
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats((0.625f+tempf)*fireVal, 0.625f*fireVal),
				 col);

	if(fireVal > 1.0f)
		fireVal -= 0.1f;
}

//------------------------------------------------------------------------------
void Player::mouseMove(const TwoFloats& pos)
{
	lastMouse.x = pos.x;
}

//------------------------------------------------------------------------------
void Player::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"VaderBullet") ||
	   (other == L"VaderBulletVuln") ||
	   (other == L"VaderBulletOwn") ||
	   (other == L"BlobVader"))
	{
		dead = true;
	}
}

//------------------------------------------------------------------------------
void Player::fired()
{
	fireVal = 2.0f;
}
