//	Player.h - The player ship.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PLAYER_H_
#define PLAYER_H_

#include "ThreeFloats.h"
#include "Collidable.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"

class Drawer;
typedef shared_ptr<Drawer> DrawerPtr;

///	The player ship.
class Player : public Collidable
{
  public:
	///	Constructor.
	Player();
	/// Destructor.
	~Player();

	///	Moves the player.
	void update();
	///	Draws the player.
	void draw(DrawerPtr d, const ThreeFloats& col);

	///	Updates the mouse position.
	void mouseMove(const TwoFloats& pos);

	///	Used to scale the player up when they fire.
	void fired();
	
	///	Returns the actor's collision radius.
	float getRadius() const {return 24.0f;};
	///	Returns the player's current position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the player's current speed.
	float getSpeed() const {return speed;};

	///	Returns the actor's type name.
	const std::wstring getName() const {return L"Player";};
	///	Called when the actor has collided with another.
	void collided(const std::wstring& other, const TwoFloats& otherPos);
	///	Returns true if this actor should not be counted in collisions.
	bool doNotCount() const {return false;};

	///	Returns true if the player is dead.
	bool getDead() const {return dead;};
  private:
	///	The player's position.
	TwoFloats position;

	///	The last mouse position.
	TwoFloats lastMouse;
	///	The player's current speed.
	float speed;

	///	Used to scale the player up when they fire.
	float fireVal;

	///	True if the player is dead.
	bool dead;
};

#endif
