//	PlayerBullet.h - The player's bullet class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PLAYERBULLET_H_
#define PLAYERBULLET_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "Collidable.h"
#include "TwoFloats.h"

#include <vector>

class Drawer;
typedef shared_ptr<Drawer> DrawerPtr;

///	The player's bullet class.
class PlayerBullet : public Collidable
{
  public:
	///	Constructor.
	PlayerBullet(const TwoFloats& pos, const TwoFloats& dir);
	///	Destructor.
	~PlayerBullet();

	///	Moves the bullet.
	/*!
		\return True if we should shake the screen.
	 */
	bool update();
	///	Draws the bullet.
	void draw(DrawerPtr d);
	
	///	Returns the actor's collision radius.
	float getRadius() const {return 10.0f;};
	///	Returns the bullet's position.
	const TwoFloats& getPosition() const {return position;};

	///	Returns the actor's type name.
	const std::wstring getName() const {return L"PlayerBullet";};
	///	Called when the actor has collided with another.
	void collided(const std::wstring& other, const TwoFloats& otherPos);
	///	Returns true if this actor should not be counted in collisions.
	bool doNotCount() const {return false;};

	///	Returns true if the bullet is dead.
	bool getDead() const {return dead;};
  private:
	///	The bullet trail size.
	enum
	{
		TrailSize = 32
	};

	///	The bullet's position.
	TwoFloats position;
	///	The bullet's direction.
	TwoFloats direction;
	///	The bullet's current colour.
	ThreeFloats colour;

	///	For the bullet's trail.
	std::vector<TwoFloats> trailPos;
	///	For the bullet's trail.
	std::vector<float> trailSize;
	///	For the bullet's trail.
	std::vector<ThreeFloats> trailCol;
	///	For the bullet's trail.
	std::vector<float> trailAlpha;

	///	True if the bullet is dead.
	bool dead;
};

#endif
