//	PlayerShip.cpp - The player's ship for traversing the solar system.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "PlayerShip.h"
#include "Random.h"

//------------------------------------------------------------------------------
PlayerShip::PlayerShip():
positionTweener(position, position, 0.02f),
colourTweener(colour, colour, 0.01f),
paused(true)
{
	ThreeFloats tempCol;

	positionTweener.setIndex(1.0f);

	while((tempCol == 0.0f) || (tempCol == 1.0f))
	{
		tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
	}
	colour = tempCol;

	while((tempCol == 0.0f) || (tempCol == 1.0f) || (tempCol == colour))
	{
		tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
	}
	colourTweener.reset(tempCol);
}

//------------------------------------------------------------------------------
PlayerShip::~PlayerShip()
{

}

//------------------------------------------------------------------------------
bool PlayerShip::update()
{
	bool retval = true;

	if(!positionTweener.getFinished() && !paused)
	{
		positionTweener.tick();
		retval = positionTweener.getFinished();
		paused = retval;
	}

	return retval;
}

//------------------------------------------------------------------------------
void PlayerShip::draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos)
{
	float scale = 0.33f;
	const TwoFloats origin(-32.0f, -32.0f+24.0f);
	TwoFloats tempPos(position - screenPos);

	if(tempPos.isWithin(origin, d->getDrawingAreaSize()+32.0f))
	{
		if(!colourTweener.getFinished())
			colourTweener.tick();
		else
		{
			ThreeFloats tempCol;

			while((tempCol == 0.0f) || (tempCol == 1.0f) || (tempCol == colour))
			{
				tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
				tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
				tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
			}
			colourTweener.reset(tempCol);
		}

		tempPos.y -= 24.0f;
		tempPos -= 16.0f * scale;

		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

		tempPos.y -= 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

		tempPos.x += 10.0f * scale;
		tempPos.y += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x -= 20.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);

		tempPos.x -= 10.0f * scale;
		tempPos.y += 10.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
		tempPos.x += 40.0f * scale;
		d->drawImage(L"Blob", tempPos, TwoFloats(scale), colour);
	}
}

//------------------------------------------------------------------------------
void PlayerShip::setInitialPosition(const TwoFloats& pos)
{
	position = pos;
	positionTweener.reset(pos);
	positionTweener.setIndex(1.0f);
}

//------------------------------------------------------------------------------
void PlayerShip::setDestination(const TwoFloats& dest)
{
	float dist = position.getDistance(dest);

	//So the delta for the greatest distance is 0.001, and increases as the
	//distance to be travelled gets smaller.
	positionTweener.setDelta((0.001f*560.0f)/dist);
	positionTweener.reset(dest);
}

//------------------------------------------------------------------------------
void PlayerShip::setPause(bool val)
{
	paused = val;
}
