//	PlayerShip.h - The player's ship for traversing the solar system.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PLAYERSHIP_H_
#define PLAYERSHIP_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

class Drawer;

///	The player's ship for traversing the solar system.
class PlayerShip
{
  public:
	///	Constructor.
	PlayerShip();
	///	Destructor.
	~PlayerShip();

	///	Moves the ship if necessary.
	/*!
		\return True if the ship is at a halt.
	 */
	bool update();
	///	Draws the ship.
	void draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos);

	///	Sets the ship's initial position.
	void setInitialPosition(const TwoFloats& pos);
	///	Sets the ship's destination.
	void setDestination(const TwoFloats& dest);

	///	Returns the ship's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the ship's colour.
	const ThreeFloats& getColour() const {return colour;};

	///	Returns true if the ship is in motion.
	bool getShipIsMoving() const {return !positionTweener.getFinished();};

	///	Tells the ship to pause moving.
	void setPause(bool val);
  private:
	///	The ship's position.
	TwoFloats position;
	///	Tweener used to move the ship to its destination.
	STweener<TwoFloats> positionTweener;

	///	The ship's colour.
	ThreeFloats colour;
	///	Tweener used to interpolate the ship's colour.
	STweener<ThreeFloats> colourTweener;

	///	Whether or not time is paused.
	bool paused;
};

#endif
