//	SequencerCursor.cpp - Class which keeps track of our position within the
//						  16-note sequence.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SequencerCursor.h"

//------------------------------------------------------------------------------
SequencerCursor::SequencerCursor():
position(0.0f),
tempo(120.0f),
callback(0)
{

}

//------------------------------------------------------------------------------
SequencerCursor::~SequencerCursor()
{

}

//------------------------------------------------------------------------------
void SequencerCursor::audioBlock(int numSamples)
{
	//num samples in sequence = (sRate)
	//							(-----)
	//							((bpm)) * numBeats
	//							((---))
	//							((60 ))

	//increment = ( numSteps )
	//			  (----------)
	//			  (numSamples)

	const float inc = 16.0f/((44100.0f/(tempo/60.0f))*4.0f);

	int lastStep = (int)position;
	int newStep;
	int i;

	for(i=0;i<numSamples;++i)
	{
		position += inc;
		if(position >= 16.0f)
			position -= 16.0f;

		newStep = (int)position;

		if((newStep != lastStep) && callback)
		{
			callback->nextStepCallback(newStep, i);
			lastStep = newStep;
		}
	}
}

//------------------------------------------------------------------------------
void SequencerCursor::resetPosition()
{
	const float inc = 16.0f/((44100.0f/(tempo/60.0f))*4.0f);

	position = 16.0f - inc;
}

//------------------------------------------------------------------------------
void SequencerCursor::setTempo(float val)
{
	tempo = val;
}

//------------------------------------------------------------------------------
void SequencerCursor::registerCallback(Callback *c)
{
	callback = c;
}
