//	Sun.cpp - The sun.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "Random.h"
#include "Sun.h"

//------------------------------------------------------------------------------
Sun::Sun():
scale(1.0f),
mouseIsOver(false)
{
	int i;

	for(i=0;i<NumBlobs;++i)
	{
		blobs[i].index = (float)i/(float)NumBlobs;
		blobs[i].offset = Random::getGlobal().getRandomFloat();
		blobs[i].offsetSpeed = 0.001f + (Random::getGlobal().getRandomFloat() * 0.009f);
	}
}

//------------------------------------------------------------------------------
Sun::~Sun()
{

}

//------------------------------------------------------------------------------
void Sun::draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos)
{
	int i;
	float tempf;
	float offset;
	TwoFloats tempPos;
	const TwoFloats origin(-48.0f);
	const float offsetAmp = (1.0f/(float)NumBlobs) * 0.5f;
	const TwoFloats centre = d->getDrawingAreaSize() * 0.5f;

	if(mouseIsOver && (scale < 1.25f))
		scale += 0.005f;
	else if(!mouseIsOver && (scale > 1.0f))
		scale -= 0.005f;

	tempPos = centre - 16.0f - screenPos;

	if(tempPos.isWithin(origin, d->getDrawingAreaSize()+96.0f))
	{
		d->setBlendMode(Drawer::Additive);

		for(i=0;i<NumBlobs;++i)
		{
			//Slowly rotate all blobs equally.
			blobs[i].index += 0.0005f;
			if(blobs[i].index > 1.0f)
				blobs[i].index -= 1.0f;

			//Update the blob's offset.
			blobs[i].offset += blobs[i].offsetSpeed;
			if(blobs[i].offset > 1.0f)
				blobs[i].offset -= 1.0f;

			//Calculate the actual offset value.
			offset = sinf(blobs[i].offset * 2.0f * NIALL_PI) * offsetAmp;

			//Calculate the blob's position.
			tempf = (blobs[i].index + offset) * 2.0f * NIALL_PI;
			tempPos.x = sinf(tempf) * 12.0f * scale;
			tempPos.y = -cosf(tempf) * 12.0f * scale;

			//Draw the blob.
			d->drawImage(L"Blob",
						 (centre + tempPos) - (16.0f*scale) - screenPos,
						 TwoFloats(scale),
						 ThreeFloats(0.75f,0.75f, 0.25f));
		}
	
		d->setBlendMode(Drawer::AlphaBlend);
	}
}

//------------------------------------------------------------------------------
void Sun::mouseOver(bool val)
{
	mouseIsOver = val;
}
