//	Sun.h - The sun.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SUN_H_
#define SUN_H_

#include "c0xHeaders.h"

class Drawer;

///	The sun.
class Sun
{
  public:
	///	Constructor.
	Sun();
	///	Destructor.
	~Sun();

	///	Draws the sun.
	void draw(shared_ptr<Drawer>& d, const TwoFloats& screenPos);

	///	So we know whether the mouse is over us or not.
	void mouseOver(bool val);
  private:
	///	Represents a single Blob used to draw the sun.
	struct Blob
	{
		///	The blob's index in the sine function.
		float index;
		///	The blob's offset in the sine function.
		float offset;
		///	The speed at which the blob's offset changes.
		float offsetSpeed;
	};
	///	The number of Blobs which make up the sun.
	enum
	{
		NumBlobs = 12
	};

	///	The Blobs which make up the sun.
	Blob blobs[NumBlobs];

	///	The sun's current scale.
	float scale;
	///	If the mouse is currently over the sun.
	bool mouseIsOver;
};

#endif
