//	TitleScreen.cpp - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BlobVaderSounder.h"
#include "drawing/Drawer.h"
#include "BlobGlobalData.h"
#include "TitleScreen.h"

using namespace std;

//------------------------------------------------------------------------------
GameStateCreator<TitleScreen> TitleScreen::creator;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application& app, SounderPtr& sound):
GameState(app, sound)
{
	globalData = dynamic_pointer_cast<BlobGlobalData>(getGlobalData());

	dynamic_pointer_cast<BlobVaderSounder>(sound)->loadPlanetPitches();
}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::draw(DrawerPtr& d)
{
	TwoFloats tempPos;

	d->fillRect(TwoFloats(0.0f), d->getDrawingAreaSize(), ThreeFloats(0.1f));

	d->drawImage(L"TitleText", TwoFloats(40.0f, 30.0f));

	d->fillRect(TwoFloats(20.0f, 76.0f+30.0f), TwoFloats(1220.0f, 1.0f), ThreeFloats(1.0f));
	d->fillRect(TwoFloats(20.0f, 254.0f+30.0f), TwoFloats(1220.0f, 1.0f), ThreeFloats(1.0f));
	d->fillRect(TwoFloats(20.0f, 324.0f+30.0f), TwoFloats(1220.0f, 1.0f), ThreeFloats(1.0f));
	d->fillRect(TwoFloats(20.0f, 440.0f+30.0f), TwoFloats(1220.0f, 1.0f), ThreeFloats(1.0f));
	d->fillRect(TwoFloats(20.0f, 618.0f+30.0f), TwoFloats(1220.0f, 1.0f), ThreeFloats(1.0f));

	if(globalData->getFlashingBackground())
		d->drawImage(L"TitleX", TwoFloats(40.0f, 344.0f+30.0f));
	if(globalData->getScreenShake())
		d->drawImage(L"TitleX", TwoFloats(40.0f, 391.0f+30.0f));

	tempPos = mousePos;
	d->drawImage(L"Blob", tempPos, TwoFloats(0.5f));
	tempPos.x += 10.0f;
	tempPos.y += 5.0f;
	d->drawImage(L"Blob", tempPos, TwoFloats(0.5f));
	tempPos.x -= 10.0f;
	tempPos.y += 5.0f;
	d->drawImage(L"Blob", tempPos, TwoFloats(0.5f));
	tempPos.y += 10.0f;
	d->drawImage(L"Blob", tempPos, TwoFloats(0.5f));
	tempPos.x += 18.0f;
	tempPos.y -= 11.0f;
	d->drawImage(L"Blob", tempPos, TwoFloats(0.5f));
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const wstring& action,
						  const wstring& keyPressed,
						  int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void TitleScreen::mouseDown(int button, const TwoFloats& pos)
{
	if((pos.y > (336.0f+30.0f)) && (pos.y < (384.0f+30.0f)))
		globalData->setFlashingBackground(!globalData->getFlashingBackground());
	else if((pos.y > (384.0f+30.0f)) && (pos.y < (424.0f+30.0f)))
		globalData->setScreenShake(!globalData->getScreenShake());
	else
	{
		globalData->resetGame();
		globalData->getGameCounter().reset();
		nextState(L"SolarSystemState");
	}
}

//------------------------------------------------------------------------------
void TitleScreen::mouseMove(const TwoFloats& pos)
{
	mousePos = pos;
}
