//	TitleScreen.h - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TITLESCREEN_H_
#define TITLESCREEN_H_

#include "GameState.h"
#include "TwoFloats.h"

class BlobGlobalData;
typedef shared_ptr<BlobGlobalData> BlobGlobalDataPtr;

///	The title screen state.
class TitleScreen : public GameState
{
  public:
	///	Constructor.
	TitleScreen(Application& app, SounderPtr& sound);
	///	Destructor.
	~TitleScreen();
	
	///	Draws the state.
	void draw(DrawerPtr& d);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	
	///	Called when a mouse down event is received.
	void mouseDown(int button, const TwoFloats& pos);
	///	Used to test connected line drawing.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"TitleScreen";};
	///	Used to construct instances of this state.
	static GameStateCreator<TitleScreen> creator;
  private:
	///	The current mouse position.
	TwoFloats mousePos;
	
	///	The global data for the game.
	BlobGlobalDataPtr globalData;
};

#endif
