//	VaderBullet.h - The BlobVader's bullet class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VADERBULLET_H_
#define VADERBULLET_H_

#include "TwoFloats.h"
#include "c0xHeaders.h"
#include "ThreeFloats.h"

#include <vector>

class Drawer;
typedef shared_ptr<Drawer> DrawerPtr;

///	The player's bullet class.
class VaderBullet
{
  public:
	///	Constructor.
	VaderBullet(const TwoFloats& pos);
	///	Destructor.
	~VaderBullet();

	///	Moves the bullet.
	/*!
		\return True if we should shake the screen.
	 */
	void update();
	///	Draws the bullet.
	void draw(DrawerPtr d);

	///	Returns the bullet's position.
	const TwoFloats& getPosition() const {return position;};
  private:
	///	The bullet trail size.
	enum
	{
		TrailSize = 32
	};

	///	The bullet's position.
	TwoFloats position;
	///	The bullet's current colour.
	ThreeFloats colour;

	///	For the bullet's trail.
	std::vector<TwoFloats> trailPos;
	///	For the bullet's trail.
	std::vector<float> trailSize;
	///	For the bullet's trail.
	std::vector<ThreeFloats> trailCol;
	///	For the bullet's trail.
	std::vector<float> trailAlpha;

	///	Used to give the bullet a sine trail.
	float sineIndex;
	///	Used to give the bullet a sine trail.
	float sineCentre;
};

#endif
