//	VaderDamageAction.h - Base class for any actions taken when the vader gets
//						  hit.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VADERDAMAGEACTION_H_
#define VADERDAMAGEACTION_H_

#include "NewGamePlayState.h"
#include "c0xHeaders.h"

class Drawer;
class Vader;

//------------------------------------------------------------------------------
///	Base class for any actions taken when the vader gets hit.
class VaderDamageAction
{
  public:
	///	Constructor.
	VaderDamageAction() {};
	///	Destructor.
	virtual ~VaderDamageAction() {};

	///	Called when the owning vader takes damage.
	virtual void damageTaken(Vader& vader, NewGamePlayState& state) = 0;

	///	Returns a description of what the action does.
	virtual const std::wstring getDescription() const = 0;
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderDamageAction.
class VaderDamageActionCreatorBase
{
  public:
	///	Constructor.
	VaderDamageActionCreatorBase(int difficultyLevel):
	difficulty(difficultyLevel)
	{
	
	};
	///	Destructor.
	virtual ~VaderDamageActionCreatorBase() {};

	///	Returns a new instance of a GameState subclass.
	virtual shared_ptr<VaderDamageAction> createInstance() const = 0;

	///	Returns this damage action's difficulty level.
	int getDifficulty() const {return difficulty;};
  private:
	///	This damage action's difficulty level.
	int difficulty;
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderDamageAction.
template <class T>
class VaderDamageActionCreator : public VaderDamageActionCreatorBase
{
  public:
	///	Constructor.
	VaderDamageActionCreator(int difficultyLevel):
	VaderDamageActionCreatorBase(difficultyLevel)
	{
	
	};
	///	Destructor.
	virtual ~VaderDamageActionCreator() {};

	///	Returns a new instance of a GameState subclass.
	shared_ptr<VaderDamageAction> createInstance() const
	{
		return shared_ptr<VaderDamageAction>(new T());
	};
};

#endif
