//	VaderExplosion.cpp - What happens when a Vader dies.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "VaderExplosion.h"
#include "MiscHelper.h"
#include "Random.h"

//------------------------------------------------------------------------------
VaderExplosion::VaderExplosion(const Array2D<bool, 7, 7>& blobsArray,
							   const TwoFloats& pos,
							   const ThreeFloats& col,
							   float wingMotion):
colour(col),
alpha(1.0f),
speed(6.0f),
scale(0.625f),
alphaTweener(alpha, 0.0f, 0.02f)
{
	int x, y;
	TwoFloats tempPos;
	TwoFloats tempDir;
	const float tempf = sinf(wingMotion*2.0f*NIALL_PI);

	for(y=0;y<7;++y)
	{
		for(x=0;x<7;++x)
		{
			if(blobsArray[y][x])
			{
				//i.e.	  centre   - top left of blob image     - top left of blob array
				tempPos = pos - 30.0f;
				tempPos.x += 10.0f * (float)x;
				tempPos.y += 10.0f * (float)y;

				//Add in wing motion.
				if(x < 3)
					tempPos.y += (tempf * ((float)(3-x)/3.0f)) * 8.0f;
				else if(x > 3)
					tempPos.y += (tempf * ((float)(x-3)/3.0f)) * 8.0f;

				if(tempPos != pos)
					tempDir = (tempPos - pos).normal();
				else
					tempDir = TwoFloats(0.0f, -1.0f);

				blobs.push_back(Blob());
				blobs.back().position = tempPos;
				blobs.back().direction = tempDir;
				blobs.back().speed = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
			}
		}
	}
}

//------------------------------------------------------------------------------
VaderExplosion::~VaderExplosion()
{

}

//------------------------------------------------------------------------------
bool VaderExplosion::update()
{
	unsigned int i;
	bool retval = false;
	const TwoFloats gravity(0.0f, 1.0f);

	for(i=0;i<blobs.size();++i)
	{
		if(blobs[i].direction != gravity)
		{
			blobs[i].direction += gravity * 0.1f;
			blobs[i].direction = blobs[i].direction.normal();
		}

		blobs[i].position += blobs[i].direction * speed * blobs[i].speed;
	}

	alphaTweener.tick();
	retval = alphaTweener.getFinished();

	if(speed > 2.0f)
		speed -= 0.05f;

	if(scale < 1.5f)
		scale += 0.01f;

	return retval;
}

//------------------------------------------------------------------------------
void VaderExplosion::draw(shared_ptr<Drawer>& d)
{
	unsigned int i;

	for(i=0;i<blobs.size();++i)
	{
		d->drawImage(L"Blob",
					 blobs[i].position-(16.0f*0.625f),
					 scale,
					 colour,
					 alpha);
	}
}
