//	VaderExplosion.h - What happens when a Vader dies.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VADEREXPLOSION_H_
#define VADEREXPLOSION_H_

#include "ThreeFloats.h"
#include "TwoFloats.h"
#include "Tweener.h"
#include "Array2D.h"

#include <vector>

class Drawer;

///	What happens when a Vader dies.
class VaderExplosion
{
  public:
	///	Constructor.
	/*!
		\param blobsArray The array of blobs which make up the Vader.
		\param pos The Vader's position.
		\param col The Vader's colour.
		\param wingMotion The Vader's wing motion.
	 */
	VaderExplosion(const Array2D<bool, 7, 7>& blobsArray,
				   const TwoFloats& pos,
				   const ThreeFloats& col,
				   float wingMotion);
	///	Destructor.
	~VaderExplosion();

	///	Moves the explosion's blobs.
	/*!
		\return True when the explosion is finished.
	 */
	bool update();
	///	Draws the explosion's blobs.
	void draw(shared_ptr<Drawer>& d);
  private:
	///	Struct representing a single blob.
	struct Blob
	{
		///	The blob's position.
		TwoFloats position;
		///	The blob's direction.
		TwoFloats direction;
		///	The blob's speed.
		float speed;
	};

	///	The explosion's blobs.
	std::vector<Blob> blobs;

	///	The explosion's colour.
	ThreeFloats colour;
	///	The explosion's alpha.
	float alpha;
	///	The explosion's speed.
	/*!
		So the blobs move faster at the start of the explosion.
	 */
	float speed;
	///	The blobs' size.
	float scale;

	///	The explosion's alpha tweener.
	STweener<float> alphaTweener;
};

#endif
