//	VaderKillAction.h - Base class for any actions taken when the vader gets
//						killed.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VADERKILLACTION_H_
#define VADERKILLACTION_H_

#include "NewGamePlayState.h"
#include "c0xHeaders.h"

class Drawer;
class Vader;

//------------------------------------------------------------------------------
///	Base class for any actions taken when the vader gets killed.
class VaderKillAction
{
  public:
	///	Constructor.
	VaderKillAction() {};
	///	Destructor.
	virtual ~VaderKillAction() {};

	///	Called when the owning vader dies.
	virtual void killed(Vader& vader, NewGamePlayState& state) = 0;

	///	Returns a description of what the action does.
	virtual const std::wstring getDescription() const = 0;
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderKillAction.
class VaderKillActionCreatorBase
{
  public:
	///	Constructor.
	VaderKillActionCreatorBase(int difficultyLevel):
	difficulty(difficultyLevel)
	{
	
	};
	///	Destructor.
	virtual ~VaderKillActionCreatorBase() {};

	///	Returns a new instance of a GameState subclass.
	virtual shared_ptr<VaderKillAction> createInstance() const = 0;

	///	Returns this damage action's difficulty level.
	int getDifficulty() const {return difficulty;};
  private:
	///	This damage action's difficulty level.
	int difficulty;
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderKillAction.
template <class T>
class VaderKillActionCreator : public VaderKillActionCreatorBase
{
  public:
	///	Constructor.
	VaderKillActionCreator(int difficultyLevel):
	VaderKillActionCreatorBase(difficultyLevel)
	{
	
	};
	///	Destructor.
	virtual ~VaderKillActionCreator() {};

	///	Returns a new instance of a GameState subclass.
	shared_ptr<VaderKillAction> createInstance() const
	{
		return shared_ptr<VaderKillAction>(new T());
	};
};

#endif
