//	VaderProjectile.h - Base class for any projectiles the vaders may fire.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef VADERPROJECTILE_H_
#define VADERPROJECTILE_H_

#include "Collidable.h"
#include "c0xHeaders.h"

class NewGamePlayState;
class Drawer;

//------------------------------------------------------------------------------
///	Base class for any projectiles the vaders may fire
class VaderProjectile : public Collidable
{
  public:
	///	Constructor.
	VaderProjectile(const TwoFloats& pos) {};
	///	Destructor.
	virtual ~VaderProjectile() {};

	///	Moves the projectile.
	virtual void update(const TwoFloats& playerPos) = 0;
	///	Draws the projectile.
	virtual void draw(shared_ptr<Drawer>& d) = 0;

	///	Returns true when the projectile can be deleted.
	virtual bool getDead() const = 0;
	///	Action to be carried out when the projectile is killed.
	virtual void deathAction(NewGamePlayState& state) {};
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderProjectile.
class VaderProjectileCreatorBase
{
  public:
	///	Constructor.
	VaderProjectileCreatorBase(int difficultyLevel,
							   const std::wstring& projName,
							   const std::wstring& projDesc):
	difficulty(difficultyLevel),
	name(projName),
	description(projDesc)
	{
		
	};
	///	Destructor.
	virtual ~VaderProjectileCreatorBase() {};

	///	Returns a new instance of a GameState subclass.
	virtual shared_ptr<VaderProjectile> createInstance(const TwoFloats& pos) const = 0;

	///	Returns the projectile's difficulty level.
	int getDifficulty() const {return difficulty;};
	///	Returns the projectile's name.
	const std::wstring& getName() const {return name;};
	///	Returns the projectile's description.
	const std::wstring& getDescription() const {return description;};
  private:
	///	The projectile's difficulty level.
	int difficulty;
	///	The projectile's name.
	std::wstring name;
	///	The projectile's description.
	std::wstring description;
};

//------------------------------------------------------------------------------
///	Used to create instances of a VaderProjectile.
template <class T>
class VaderProjectileCreator : public VaderProjectileCreatorBase
{
  public:
	///	Constructor.
	VaderProjectileCreator(int difficultyLevel,
						   const std::wstring& projName = L"",
						   const std::wstring& projDesc = L""):
	VaderProjectileCreatorBase(difficultyLevel, projName, projDesc)
	{
	
	};
	///	Destructor.
	virtual ~VaderProjectileCreator() {};

	///	Returns a new instance of a GameState subclass.
	shared_ptr<VaderProjectile> createInstance(const TwoFloats& pos) const
	{
		return shared_ptr<VaderProjectile>(new T(pos));
	};
};

#endif
