//	EncyclopediaInfoBox.cpp - The Vader Encyclopedia info box.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "VaderDamageAction.h"
#include "VaderProjectile.h"
#include "EncyclopediaBox.h"
#include "VaderKillAction.h"
#include "drawing/Drawer.h"
#include "BlobGlobalData.h"
#include "Random.h"
#include "Vader.h"

//------------------------------------------------------------------------------
EncyclopediaInfoBox::EncyclopediaInfoBox(shared_ptr<BlobGlobalData>& data,
										 const TwoFloats& drawSize):
globalData(data),
currentVader(0),
blobColourLeft(1.0f, 0.0f, 0.0f),
blobLeftTweener(blobColourLeft, ThreeFloats(0.0f, 0.0f, 1.0f), 0.01f),
blobColourRight(0.0f, 0.0f, 1.0f),
blobRightTweener(blobColourRight, ThreeFloats(0.0f, 1.0f, 0.0f), 0.01f),
infoPos(((drawSize.x*0.5f) - 211.0f), ((drawSize.y*0.5f) - 261.5f)),
infoSize(422.0f, 523.0f)
{
	if(globalData->getEncyclopediaSize() > 0)
		updateVader();
}

//------------------------------------------------------------------------------
EncyclopediaInfoBox::~EncyclopediaInfoBox()
{

}

//------------------------------------------------------------------------------
void EncyclopediaInfoBox::draw(shared_ptr<Drawer>& d)
{
	if(!blobRightTweener.getFinished())
		blobRightTweener.tick();
	else
	{
		ThreeFloats tempCol;

		while((tempCol == 0.0f) || (tempCol == 1.0f) || (tempCol == blobColourRight))
		{
			tempCol.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
			tempCol.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
			tempCol.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : 0.0f;
		}
		blobRightTweener.reset(tempCol);
	}

	if(!blobLeftTweener.getFinished())
		blobLeftTweener.tick();
	else
		blobLeftTweener.reset(blobColourRight);

	d->drawText(L"Vader Encyclopedia",
				L"Font1",
				layouts[Encyclopedia],
				TwoFloats(infoPos.x+12.0f, infoPos.y+41.0f),
						  (infoSize.x-24.0f-31.0f),
						  TextLayout::Centre,
						  1.0f,
						  ThreeFloats(1.0f));

	d->drawText(L"<",
				L"Font1",
				layouts[LeftArrow],
				TwoFloats(infoPos.x+16.0f, infoPos.y+(infoSize.y*0.5f)),
				48.0f,
				TextLayout::Left,
				1.0f,
				ThreeFloats(1.0f));
	d->drawText(L">",
				L"Font1",
				layouts[RightArrow],
				TwoFloats(infoPos.x+infoSize.x-16.0f-48.0f, infoPos.y+(infoSize.y*0.5f)),
				48.0f,
				TextLayout::Right,
				1.0f,
				ThreeFloats(1.0f));

	if(globalData->getEncyclopediaSize() > 0)
	{
		const shared_ptr<Vader>& tempVader = globalData->getEncyclopediaVader(currentVader);
		d->drawText(tempVader->getVaderName(),
					L"Font1",
					layouts[VaderName],
					TwoFloats(infoPos.x+24.0f+24.0f, infoPos.y+64.0f+40.0f),
					infoSize.x-32.0f-64.0f,
					TextLayout::Left,
					1.0f,
					ThreeFloats(1.0f));

		tempVader->draw(d, blobColourLeft, blobColourRight);

		if(descriptionText.length() > 0)
		{
			d->drawText(descriptionText,
						L"Font2",
						layouts[VaderDescription],
						TwoFloats(infoPos.x+24.0f+24.0f, infoPos.y+208.0f+40.0f),
						infoSize.x-32.0f-64.0f,
						TextLayout::Left,
						1.0f,
						ThreeFloats(1.0f));
		}
	}
	else
	{
		d->drawText(L"<no data>",
					L"Font1",
					layouts[VaderName],
					TwoFloats(infoPos.x+24.0f+24.0f, infoPos.y+64.0f+40.0f),
					infoSize.x-32.0f-64.0f,
					TextLayout::Left,
					1.0f,
					ThreeFloats(1.0f));
	}
}

//------------------------------------------------------------------------------
void EncyclopediaInfoBox::mouseDown(int button, const TwoFloats& pos)
{
	TwoFloats tempPos(infoPos);
	TwoFloats tempSize(infoSize);
	const TwoFloats buttonSize(48.0f, infoSize.y);

	tempPos.x += (infoSize.x-48.0f);
	tempPos.y += 32.0f;
	tempSize.y -= 32.0f;
	if(pos.isWithin(infoPos, buttonSize))
	{
		if(globalData->getEncyclopediaSize() > 0)
		{
			--currentVader;
			if(currentVader < 0)
				currentVader = globalData->getEncyclopediaSize()-1;

			updateVader();
		}
	}
	else if(pos.isWithin(tempPos, tempSize))
	{
		if(globalData->getEncyclopediaSize() > 0)
		{
			++currentVader;
			currentVader %= globalData->getEncyclopediaSize();

			updateVader();
		}
	}
}

//------------------------------------------------------------------------------
void EncyclopediaInfoBox::updateVader()
{
	unsigned int i;
	const shared_ptr<Vader>& vader = globalData->getEncyclopediaVader(currentVader);
	const std::vector<shared_ptr<VaderProjectileCreatorBase> >& projectiles = vader->getProjectileCreators();
	const std::vector<shared_ptr<VaderDamageAction> >& damages = vader->getDamageActions();
	const shared_ptr<VaderKillAction>& kill = vader->getKillAction();

	layouts[VaderName].clear();
	layouts[VaderDescription].clear();

	vader->setPosition(TwoFloats(infoPos.x+80.0f+24.0f, infoPos.y+176.0f));

	descriptionText = L"";

	if(projectiles.size() > 0)
	{
		descriptionText = L"Shoots:\n";

		for(i=0;i<projectiles.size();++i)
		{
			descriptionText.append(projectiles[i]->getName());
			descriptionText.append(L": ");
			descriptionText.append(projectiles[i]->getDescription());
			descriptionText.append(L"\n");
		}
		descriptionText.append(L"\n");
	}

	if(damages.size() > 0)
	{
		descriptionText.append(L"When taking damage:\n");

		for(i=0;i<damages.size();++i)
		{
			descriptionText.append(damages[i]->getDescription());
			descriptionText.append(L"\n");
		}
		descriptionText.append(L"\n");
	}

	if(kill)
	{
		descriptionText.append(L"When it dies:\n");
		descriptionText.append(vader->getKillAction()->getDescription());
	}

	if(descriptionText == L"")
		descriptionText = L"This vader is not particularly interesting.";
}
