//	EncyclopediaBox.h - The Vader Encyclopedia info box.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ENCYCLOPEDIABOX_H_
#define ENCYCLOPEDIABOX_H_

#include "drawing/TextLayout.h"
#include "ThreeFloats.h"
#include "../InfoBox.h"
#include "Tweener.h"

class BlobGlobalData;

///	The Vader Encyclopedia info box.
class EncyclopediaInfoBox : public InfoBox
{
  public:
	///	Constructor.
	EncyclopediaInfoBox(shared_ptr<BlobGlobalData>& data,
						const TwoFloats& drawSize);
	///	Destructor.
	~EncyclopediaInfoBox();

	///	Draws the info box.
	void draw(shared_ptr<Drawer>& d);

	///	Handles mouse moves.
	void mouseMove(const TwoFloats& pos) {};
	///	Handles mouse downs.
	void mouseDown(int button, const TwoFloats& pos);
  private:
	///	Helper method which updates the vader info.
	void updateVader();

	///	Constants representing the various text layouts.
	enum
	{
		Encyclopedia = 0,
		VaderName,
		VaderDescription,
		LeftArrow,
		RightArrow,

		NumLayouts
	};

	///	Our copy of the global data.
	shared_ptr<BlobGlobalData> globalData;
	///	Which vader we're currently displaying.
	int currentVader;
	///	The description of the current vader.
	std::wstring descriptionText;

	///	Used to colour cycle the vader.
	ThreeFloats blobColourLeft;
	///	Used to colour cycle the vader.
	STweener<ThreeFloats> blobLeftTweener;
	///	Used to colour cycle the vader.
	ThreeFloats blobColourRight;
	///	Used to colour cycle the vader.
	STweener<ThreeFloats> blobRightTweener;
	
	///	The position of the info box.
	TwoFloats infoPos;
	///	The size of the info box.
	TwoFloats infoSize;

	///	The TextLayouts used to render the text.
	TextLayout layouts[NumLayouts];
};

#endif
