//	PlanetBox.h - The Planet info box.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PLANETBOX_H_
#define PLANETBOX_H_

#include "drawing/TextLayout.h"
#include "../InfoBox.h"
#include "../Planet.h"

///	The Planet info box.
class PlanetInfoBox : public InfoBox
{
  public:
	///	Constructor.
	PlanetInfoBox(shared_ptr<Planet>& p, const TwoFloats& drawSize);
	///	Destructor.
	~PlanetInfoBox();

	///	Draws the info box.
	void draw(shared_ptr<Drawer>& d);

	///	Handles mouse moves.
	void mouseMove(const TwoFloats& pos) {};
	///	Handles mouse downs.
	void mouseDown(int button, const TwoFloats& pos) {};

	///	Sets the planet this box is describing.
	void setPlanet(shared_ptr<Planet>& p);
  private:
	///	Constants representing the various text layouts.
	enum
	{
		PlanetName = 0,
		PlanetDescription,

		NumLayouts
	};

	///	The selected planet.
	shared_ptr<Planet> planet;
	
	///	The position of the info box.
	TwoFloats infoPos;
	///	The size of the info box.
	TwoFloats infoSize;

	///	The TextLayouts used to render the text.
	TextLayout layouts[NumLayouts];
};

#endif
