//	BasicBullet.cpp - The basic vader bullet class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "BasicBullet.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
BasicBullet::BasicBullet(const TwoFloats& pos):
VaderProjectile(pos),
position(pos),
colour(ThreeFloats(0.0f, 1.0f, 1.0f)),
sineIndex(0.0f),
sineCentre(position.x),
dead(false)
{
	int i;

	for(i=0;i<TrailSize;++i)
	{
		trailPos.push_back(position);
		trailSize.push_back(fakeLog((float)(i)/(float)TrailSize) * 8.0f);
		trailCol.push_back(colour);
		trailAlpha.push_back((float)(i)/(float)TrailSize);
	}
}

//------------------------------------------------------------------------------
BasicBullet::~BasicBullet()
{

}

//------------------------------------------------------------------------------
void BasicBullet::update(const TwoFloats& playerPos)
{
	float tempf;
	const ThreeFloats topColour(0.0f, 1.0f, 1.0f);
	const ThreeFloats bottomColour(0.0f, 0.0f, 1.0f);

	position.y += 8.0f;
	position.x = sineCentre + (sinf(sineIndex * 2.0f * NIALL_PI)*4.0f);
	tempf = position.y/640.0f;
	colour = (topColour * (1.0f-tempf)) + (bottomColour * tempf);

	trailPos.erase(trailPos.begin());
	trailPos.push_back(position);
	trailCol.erase(trailCol.begin());
	trailCol.push_back(colour);

	sineIndex += 0.1f;
	if(sineIndex > 1.0f)
		sineIndex -= 1.0f;

	if(position.y > 1024.0f)
		dead = true;
}

//------------------------------------------------------------------------------
void BasicBullet::draw(shared_ptr<Drawer>& d)
{
	TwoFloats origin;
	TwoFloats tempPos;

	d->drawConnectedLine(L"",
						 trailPos,
						 trailSize,
						 trailCol,
						 trailAlpha);
	tempPos = trailPos[TrailSize-3];
	tempPos.x -= (16.0f*0.625f);
	tempPos.y -= (16.0f*1.0f);
	d->drawImage(L"Blob",
				 tempPos,
				 TwoFloats(0.625f, 1.0f),
				 colour);
}

//------------------------------------------------------------------------------
void BasicBullet::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if(other == L"Player")
		dead = true;
}
