//	ExplodeBullet.cpp - The basic vader bullet class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "ExplodeBullet.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
ExplodeBullet::ExplodeBullet(const TwoFloats& pos, const TwoFloats& dir):
VaderProjectile(pos),
position(pos),
direction(dir),
colour(ThreeFloats(1.0f, 0.5f, 0.0f)),
life(1.0f),
dead(false)
{
	int i;

	for(i=0;i<TrailSize;++i)
	{
		trailPos.push_back(position);
		trailSize.push_back(fakeLog((float)(i)/(float)TrailSize) * 8.0f);
		trailCol.push_back(colour);
		trailAlpha.push_back((float)(i)/(float)TrailSize);
	}
}

//------------------------------------------------------------------------------
ExplodeBullet::~ExplodeBullet()
{

}

//------------------------------------------------------------------------------
void ExplodeBullet::update(const TwoFloats& playerPos)
{
	float tempf;
	const ThreeFloats topColour(0.0f, 1.0f, 1.0f);
	const ThreeFloats bottomColour(0.0f, 0.0f, 1.0f);

	position += direction * 4.0f;
	tempf = position.y/640.0f;
	colour = (topColour * (1.0f-tempf)) + (bottomColour * tempf);

	trailPos.erase(trailPos.begin());
	trailPos.push_back(position);
	trailCol.erase(trailCol.begin());
	trailCol.push_back(colour);

	if(position.x < -128.0f)
		dead = true;
	else if(position.x > 1536.0f)
		dead = true;
	if(position.y < -128.0f)
		dead = true;
	else if(position.y > 1024.0f)
		dead = true;

	life -= 0.02f;
	if(life <= 0.0f)
		dead = true;
}

//------------------------------------------------------------------------------
void ExplodeBullet::draw(shared_ptr<Drawer>& d)
{
	TwoFloats origin;
	TwoFloats tempPos;
	float angle = -origin.getAngle(direction);

	d->drawConnectedLine(L"",
						 trailPos,
						 trailSize,
						 trailCol,
						 trailAlpha);
	tempPos = trailPos[TrailSize-3];
	tempPos.x -= (16.0f*0.625f);
	tempPos.y -= (16.0f*1.0f);
	d->drawRotImage(L"Blob",
					tempPos,
					angle,
					TwoFloats(16.0f),
					TwoFloats(0.625f, 1.0f),
					colour*1.33f);
}

//------------------------------------------------------------------------------
void ExplodeBullet::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"Player") || (other == L"BlobVader"))
		dead = true;
}
