//	GrenadeBullet.cpp - Explodes into ExplodeBullets on collision/after a set
//						time.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "NewGamePlayState.h"
#include "drawing/Drawer.h"
#include "Explode2Bullet.h"
#include "GrenadeBullet.h"
#include "MiscHelper.h"
#include "Random.h"

//------------------------------------------------------------------------------
GrenadeBullet::GrenadeBullet(const TwoFloats& pos):
VaderProjectile(pos),
position(pos),
colour(ThreeFloats(0.0f, 1.0f, 1.0f)),
speed(6.0f),
life(1.0f),
dead(false)
{
	int i;

	direction.x = Random::getGlobal().getRandomFloat() - 0.5f;
	direction.y = -1.0f;
	direction = direction.normal();

	for(i=0;i<TrailSize;++i)
	{
		trailPos.push_back(position);
		trailSize.push_back(fakeLog((float)(i)/(float)TrailSize) * 8.0f);
		trailCol.push_back(colour);
		trailAlpha.push_back((float)(i)/(float)TrailSize);
	}
}

//------------------------------------------------------------------------------
GrenadeBullet::~GrenadeBullet()
{

}

//------------------------------------------------------------------------------
void GrenadeBullet::update(const TwoFloats& playerPos)
{
	float tempf;
	const TwoFloats gravity(0.0f, 1.0f);
	const ThreeFloats topColour(1.0f, 1.0f, 0.0f);
	const ThreeFloats bottomColour(0.0f, 0.66f, 0.0f);

	if(direction != gravity)
		direction += (gravity-direction) * 0.02f;

	if(speed > 4.0f)
		speed -= 0.05f;

	position += direction * speed;
	tempf = position.y/640.0f;
	colour = (topColour * (1.0f-tempf)) + (bottomColour * tempf);

	trailPos.erase(trailPos.begin());
	trailPos.push_back(position);
	trailCol.erase(trailCol.begin());
	trailCol.push_back(colour);

	if(position.x < -128.0f)
		dead = true;
	else if(position.x > 1536.0f)
		dead = true;
	if(position.y < -128.0f)
		dead = true;
	else if(position.y > 1024.0f)
		dead = true;

	life -= 0.005f;
	if(life <= 0.0f)
		dead = true;
}

//------------------------------------------------------------------------------
void GrenadeBullet::draw(shared_ptr<Drawer>& d)
{
	TwoFloats origin;
	TwoFloats tempPos;
	float angle = -origin.getAngle(direction);

	d->drawConnectedLine(L"",
						 trailPos,
						 trailSize,
						 trailCol,
						 trailAlpha);
	tempPos = trailPos[TrailSize-3];
	tempPos.x -= (16.0f*0.8f);
	tempPos.y -= (16.0f*1.0f);
	d->drawRotImage(L"Blob",
					tempPos,
					angle,
					TwoFloats(16.0f),
					TwoFloats(0.8f, 1.0f),
					colour*1.33f);
}

//------------------------------------------------------------------------------
void GrenadeBullet::deathAction(NewGamePlayState& state)
{
	int i;
	TwoFloats dir;

	if(position.y < 720.0f)
	{
		for(i=0;i<8;++i)
		{
			dir.x = sinf(((float)i/8.0f) * 2.0f * NIALL_PI);
			dir.y = cosf(((float)i/8.0f) * 2.0f * NIALL_PI);

			state.addVaderProjectile(shared_ptr<VaderProjectile>(new Explode2Bullet(position, dir)));
		}
	}
}

//------------------------------------------------------------------------------
void GrenadeBullet::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"Player") || (other == L"PlayerBullet"))
		dead = true;
}
