//	GrenadeBullet.h - Explodes into ExplodeBullets on collision/after a set
//					  time.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GRENADEBULLET_H_
#define GRENADEBULLET_H_

#include "../VaderProjectile.h"
#include "ThreeFloats.h"

#include <vector>

///	Explodes into ExplodeBullets on collision/after a set time.
class GrenadeBullet : public VaderProjectile
{
  public:
	///	Constructor.
	GrenadeBullet(const TwoFloats& pos);
	///	Destructor.
	~GrenadeBullet();

	///	Moves the projectile.
	void update(const TwoFloats& playerPos);
	///	Draws the projectile.
	void draw(shared_ptr<Drawer>& d);

	///	Returns true when the projectile can be deleted.
	bool getDead() const {return dead;};
	///	Triggers the ExplodeBullets when the bullet dies.
	void deathAction(NewGamePlayState& state);

	///	Returns the actor's collision radius.
	float getRadius() const {return 12.0f;};
	///	Returns the actor's position.
	const TwoFloats& getPosition() const {return position;};

	///	Returns the actor's type name.
	const std::wstring getName() const {return L"VaderBulletVuln";};
	///	Called when the actor has collided with another.
	void collided(const std::wstring& other, const TwoFloats& otherPos);
	///	Returns true if this actor should not be counted in collisions.
	bool doNotCount() const {return false;};
  private:
	///	The bullet trail size.
	enum
	{
		TrailSize = 32
	};

	///	The bullet's position.
	TwoFloats position;
	///	The bullet's direction.
	TwoFloats direction;
	///	The bullet's current colour.
	ThreeFloats colour;

	///	For the bullet's trail.
	std::vector<TwoFloats> trailPos;
	///	For the bullet's trail.
	std::vector<float> trailSize;
	///	For the bullet's trail.
	std::vector<ThreeFloats> trailCol;
	///	For the bullet's trail.
	std::vector<float> trailAlpha;

	///	The bullet's speed.
	float speed;

	///	Used to ensure the bullet dies off after a while.
	float life;
	///	True if the bullet is dead.
	bool dead;
};

#endif
