//	HomingBullet.cpp - Homing bullet class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "HomingBullet.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
HomingBullet::HomingBullet(const TwoFloats& pos, const TwoFloats dir):
VaderProjectile(pos),
position(pos),
direction(dir),
colour(ThreeFloats(1.0f, 0.0f, 1.0f)),
dead(false)
{
	int i;

	for(i=0;i<TrailSize;++i)
	{
		trailPos.push_back(position);
		trailSize.push_back(fakeLog((float)(i)/(float)TrailSize) * 8.0f);
		trailCol.push_back(colour);
		trailAlpha.push_back((float)(i)/(float)TrailSize);
	}
}

//------------------------------------------------------------------------------
HomingBullet::~HomingBullet()
{

}

//------------------------------------------------------------------------------
void HomingBullet::update(const TwoFloats& playerPos)
{
	float tempf;
	const TwoFloats playerDir = (playerPos-position).normal();
	const ThreeFloats topColour(1.0f, 0.0f, 1.0f);
	const ThreeFloats bottomColour(1.0f, 0.0f, 0.0f);

	if(direction.getDistance(playerDir) < 0.01f)
		direction = playerDir;
	else if(position.y < 720.0f)
	{
		direction += (playerDir-direction) * (1.0f/48.0f);
		direction = direction.normal();
	}
	else
		direction.y = 1.0f;

	if((position.x < -4.0f) || (position.x > 1284.0f))
		direction = TwoFloats(0.0f, 1.0f);

	position += direction * 8.0f;
	tempf = position.y/640.0f;
	colour = (topColour * (1.0f-tempf)) + (bottomColour * tempf);

	trailPos.erase(trailPos.begin());
	trailPos.push_back(position);
	trailCol.erase(trailCol.begin());
	trailCol.push_back(colour);

	if(position.y > 1024.0f)
		dead = true;
}

//------------------------------------------------------------------------------
void HomingBullet::draw(shared_ptr<Drawer>& d)
{
	TwoFloats origin;
	TwoFloats tempPos;
	float angle = -origin.getAngle(direction);

	d->drawConnectedLine(L"",
						 trailPos,
						 trailSize,
						 trailCol,
						 trailAlpha);
	tempPos = trailPos[TrailSize-3];
	tempPos.x -= (16.0f*0.625f);
	tempPos.y -= (16.0f*1.0f);
	d->drawRotImage(L"Blob",
					tempPos,
					angle,
					TwoFloats(16.0f),
					TwoFloats(0.625f, 1.0f),
					colour*1.33f);
}

//------------------------------------------------------------------------------
void HomingBullet::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"Player") || (other == L"PlayerBullet"))
		dead = true;
}
