//	Mine.cpp - Exploding mine bullet.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "NewGamePlayState.h"
#include "drawing/Drawer.h"
#include "ExplodeBullet.h"
#include "MiscHelper.h"
#include "Random.h"
#include "Mine.h"

//------------------------------------------------------------------------------
Mine::Mine(const TwoFloats& pos):
VaderProjectile(pos),
position(pos),
direction(0.0f, 1.0f),
colour(ThreeFloats(0.0f, 1.0f, 1.0f)),
rotIndex(0.25f),
speed(6.0f),
life(1.0f),
halted(false),
dead(false)
{
	int i;

	for(i=0;i<TrailSize;++i)
	{
		trailPos.push_back(position);
		trailSize.push_back(fakeLog((float)(i)/(float)TrailSize) * 8.0f);
		trailCol.push_back(colour);
		trailAlpha.push_back((float)(i)/(float)TrailSize);
	}
}

//------------------------------------------------------------------------------
Mine::~Mine()
{

}

//------------------------------------------------------------------------------
void Mine::update(const TwoFloats& playerPos)
{
	float tempf;
	const TwoFloats lastPos(position);
	const ThreeFloats topColour(0.33f, 0.66f, 0.33f);
	const ThreeFloats bottomColour(0.0f, 0.0f, 1.0f);

	if(!halted)
	{
		if(speed > 4.0f)
			speed -= 0.05f;

		position += direction * speed;
		tempf = position.y/640.0f;
		colour = (topColour * (1.0f-tempf)) + (bottomColour * tempf);
	}
	else
	{
		position.x = centre.x + sinf(rotIndex * 2.0f * NIALL_PI) * 32.0f;
		position.y = centre.y + cosf(rotIndex * 2.0f * NIALL_PI) * 32.0f;
		rotIndex += 0.01f;
		if(rotIndex > 1.0f)
			rotIndex -= 1.0f;

		direction = (position - lastPos).normal();

		if(position.y > 730.0f)
			dead = true;
	}

	trailPos.erase(trailPos.begin());
	trailPos.push_back(position);
	trailCol.erase(trailCol.begin());
	trailCol.push_back(colour);

	if(position.y > 1024.0f)
		dead = true;

	if(!halted && (position.y < 620.0f))
	{
		life -= 0.015f;
		if(life <= 0.0f)
		{
			halted = true;
			centre = position;
			centre.x -= 32.0f;
		}
	}
}

//------------------------------------------------------------------------------
void Mine::draw(shared_ptr<Drawer>& d)
{
	TwoFloats origin;
	TwoFloats tempPos;
	float angle = -origin.getAngle(direction);

	d->drawConnectedLine(L"",
						 trailPos,
						 trailSize,
						 trailCol,
						 trailAlpha);
	tempPos = trailPos[TrailSize-3];
	tempPos.x -= (16.0f*0.8f);
	tempPos.y -= (16.0f*1.0f);
	d->drawRotImage(L"Blob",
					tempPos,
					angle,
					TwoFloats(16.0f),
					TwoFloats(0.8f, 1.0f),
					colour*1.33f);
}

//------------------------------------------------------------------------------
void Mine::deathAction(NewGamePlayState& state)
{
	int i;
	TwoFloats dir;

	if(position.y < 720.0f)
	{
		for(i=0;i<8;++i)
		{
			dir.x = sinf(((float)i/8.0f) * 2.0f * NIALL_PI);
			dir.y = cosf(((float)i/8.0f) * 2.0f * NIALL_PI);

			state.addVaderProjectile(shared_ptr<VaderProjectile>(new ExplodeBullet(position, dir)));
		}
	}
}

//------------------------------------------------------------------------------
void Mine::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"Player") ||
	   (other == L"PlayerBullet") ||
	   (other == L"VaderBullet") ||
	   (other == L"VaderBulletOwn") ||
	   (other == L"VaderBulletVuln"))
	{
		if(!halted)
		{
			halted = true;
			centre = position;
			centre.x -= 32.0f;
		}
		else
			dead = true;
	}
}
