//	CrashSynth.h - The crash synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef CRASHSYNTH_H_
#define CRASHSYNTH_H_

#include "Synthesizer.h"

///	The crash synthesizer.
class CrashSynth : public Synthesizer
{
  public:
	///	Constructor.
	CrashSynth();
	///	Destructor.
	~CrashSynth();

	///	Returns the audio for this synthesizer.
	void getAudio(float *buffer, int numSamples);

	///	Sets whether or not this synthesizer is active.
	void setActive(bool val, int startSample, bool hi);
	///	Returns whether or not this synthesizer is active.
	bool getActive() const {return active;};
  private:
	///	Whether or not the synthesizer is currently active.
	bool active;
	///	Which sample to start on in the next audio block.
	int start;

	///	The length of the kick sound (counts down).
	float length;
	///	The value used to count down length.
	float increment;

	///	The hi-pass filter.
	float filter;
	///	The filter coefficient.
	float filterCoeff;

	///	The size of the delayline.
	enum
	{
		DelaySize = 64
	};
	///	The delayline.
	float delay[DelaySize];
	///	The delayline feedback.
	float feedback;
	///	The delay write pointer.
	int delayWrite;
	///	The delay length.
	float delayLength;
};

#endif
