//	HatSynth.cpp - The hat synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "HatSynth.h"
#include "MiscHelper.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
HatSynth::HatSynth():
active(false),
start(0),
length(0.0f),
increment(8.0f/44100.0f),
filter(0.0f),
filterCoeff(1.0f)
{
	
}

//------------------------------------------------------------------------------
HatSynth::~HatSynth()
{
	
}

//------------------------------------------------------------------------------
void HatSynth::getAudio(float *buffer, int numSamples)
{
	int i;
	float tempf, tempf2;

	for(i=start;i<numSamples;++i)
	{
		//Count down the sound's timer.
		length -= increment;
		if(length <= 0.0f)
		{
			setActive(false, 0, false);
			break;
		}

		//Generate the audio.
		tempf2 = Random::getGlobal().getRandomFloat();
		tempf = (tempf2 - (filter * filterCoeff));
		filter = tempf2;
		buffer[i*2] += tempf * fakeExp(length) * 0.33f;
		buffer[(i*2)+1] += tempf * fakeExp(length) * 0.33f;
	}
	start = 0;
}

//------------------------------------------------------------------------------
void HatSynth::setActive(bool val, int startSample, bool hi)
{
	active = val;
	if(active)
	{
		start = startSample;

		length = 1.0f;

		if(hi)
			filterCoeff = 0.7f;
		else
			filterCoeff = 1.0f;
	}
}
