//	KickSynth.h - The kick drum synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef KICKSYNTH_H_
#define KICKSYNTH_H_

#include "Synthesizer.h"

///	The kick drum synthesizer.
class KickSynth : public Synthesizer
{
  public:
	///	Constructor.
	KickSynth();
	///	Destructor.
	~KickSynth();

	///	Returns the audio for this synthesizer.
	void getAudio(float *buffer, int numSamples);

	///	Sets whether or not this synthesizer is active.
	void setActive(bool val, int startSample, bool hi);
	///	Returns whether or not this synthesizer is active.
	bool getActive() const {return active;};
  private:
	///	Whether or not the synthesizer is currently active.
	bool active;
	///	Which sample to start on in the next audio block.
	int start;

	///	The length of the kick sound (counts down).
	float length;
	///	The value used to count down length.
	float increment;

	///	For the sine generation.
	float x0;
	///	For the sine generation.
	float x1;
	///	For the sine generation.
	float coeff;

	///	We interpolate between coefficients to save on the sin() calls.
	float hiCoeff[2];
	///	Which hiCoeff we're using.
	int hiIndex;
	///	We interpolate between coefficients to save on the sin() calls.
	float loCoeff;
};

#endif
