//	MonophonicSynth.h - Base class for a single voice of a synth.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MONOPHONICSYNTH_H_
#define MONOPHONICSYNTH_H_

#include <string>

///	Base class for a single voice of a synth.
class MonophonicSynth
{
  public:
	///	Constructor.
	MonophonicSynth():
	active(false)
	{
		
	};
	///	Destructor.
	virtual ~MonophonicSynth() {};

	///	Returns the audio for this synth.
	/*!
		\param buffer Stereo (interleaved) buffer to write audio to.
		\param numSamples The number of samples in the current block (i.e.
		buffer will contain 2x numSamples.
	 */
	virtual void getAudio(float *buffer, int numSamples) = 0;

	///	Triggers a note at the specified frequency.
	/*!
		\param freq The frequency of the note to trigger.
		\param sample The sample within the current audio block to trigger
		the note at.
	 */
	virtual void triggerNote(float freq, int sample) = 0;
	///	Sets the named parameter.
	/*!
		\param name The name of the parameter to change.
		\param value The value to change the parameter to (0->1).
		\param direct Can be used to bypass any further manipulation you may
		want to do in setParameter() (e.g. applying a log curve to the value
		passed in) and set the internal parameter directly.  Only really
		useful for saving/loading patches.
	 */
	virtual void setParameter(const std::string& name, float value, bool direct = false) = 0;
	///	Returns the number of parameters this synth has.
	virtual int getNumParameters() const = 0;
	///	Returns the value of the indexed parameter.
	virtual float getParameter(int i) const = 0;
	///	Returns the name of the indexed parameter.
	virtual std::string getParameterName(int i) const = 0;

	///	Returns whether or not this voice is active.
	bool getActive() const {return active;};
  protected:
	///	Sets whether or not this voice is active (to be called from subclasses).
	void setActive(bool val) {active = val;};
  private:
	///	Whether or not this voice is active.
	bool active;
};

#endif
